@@ -14,16 +14,25 @@ import (
 	adminv1 "github.com/acorn-io/acorn/pkg/apis/internal.admin.acorn.io/v1"
 	"github.com/acorn-io/acorn/pkg/appdefinition"
 	"github.com/acorn-io/acorn/pkg/client"
+	"github.com/acorn-io/acorn/pkg/config"
 	kclient "github.com/acorn-io/acorn/pkg/k8sclient"
 	"github.com/acorn-io/acorn/pkg/labels"
 	"github.com/acorn-io/acorn/pkg/run"
+	"github.com/acorn-io/acorn/pkg/scheme"
+	"github.com/acorn-io/acorn/pkg/tolerations"
+	"github.com/acorn-io/baaah/pkg/apply"
+	"github.com/acorn-io/baaah/pkg/restconfig"
+	"github.com/acorn-io/baaah/pkg/router"
+	"github.com/acorn-io/baaah/pkg/watcher"
+	"github.com/google/uuid"
 	"github.com/stretchr/testify/assert"
 	corev1 "k8s.io/api/core/v1"
 	storagev1 "k8s.io/api/storage/v1"
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	runtimeclient "sigs.k8s.io/controller-runtime/pkg/client"
+	k8slabels "k8s.io/apimachinery/pkg/labels"
+	crClient "sigs.k8s.io/controller-runtime/pkg/client"
 )
 
 func TestVolume(t *testing.T) {
@@ -125,6 +134,7 @@ func TestVolumeBadClassInImageBoundToGoodClass(t *testing.T) {
 	volumeClass := adminapiv1.ClusterVolumeClass{
 		ObjectMeta:       metav1.ObjectMeta{Name: "acorn-test-custom"},
 		StorageClassName: getStorageClassName(t, storageClasses),
+		SupportedRegions: []string{apiv1.LocalRegion},
 	}
 	if err = kclient.Create(ctx, &volumeClass); err != nil {
 		t.Fatal(err)
@@ -183,7 +193,9 @@ func TestVolumeBoundBadClass(t *testing.T) {
 	volumeClass := adminapiv1.ClusterVolumeClass{
 		ObjectMeta:       metav1.ObjectMeta{Name: "acorn-test-custom"},
 		StorageClassName: getStorageClassName(t, storageClasses),
+		SupportedRegions: []string{apiv1.LocalRegion},
 	}
+
 	if err := kclient.Create(ctx, &volumeClass); err != nil {
 		t.Fatal(err)
 	}
@@ -221,8 +233,9 @@ func TestVolumeClassInactive(t *testing.T) {
 	c, _ := helper.ClientAndNamespace(t)
 
 	volumeClass := adminapiv1.ClusterVolumeClass{
-		ObjectMeta: metav1.ObjectMeta{Name: "acorn-test-custom"},
-		Inactive:   true,
+		ObjectMeta:       metav1.ObjectMeta{Name: "acorn-test-custom"},
+		Inactive:         true,
+		SupportedRegions: []string{apiv1.LocalRegion},
 	}
 	if err := kclient.Create(ctx, &volumeClass); err != nil {
 		t.Fatal(err)
@@ -259,6 +272,7 @@ func TestVolumeClassSizeTooSmall(t *testing.T) {
 			Min: "10Gi",
 			Max: "100Gi",
 		},
+		SupportedRegions: []string{apiv1.LocalRegion},
 	}
 	if err := kclient.Create(ctx, &volumeClass); err != nil {
 		t.Fatal(err)
@@ -302,6 +316,7 @@ func TestVolumeClassSizeTooLarge(t *testing.T) {
 			Min: "10Gi",
 			Max: "100Gi",
 		},
+		SupportedRegions: []string{apiv1.LocalRegion},
 	}
 	if err := kclient.Create(ctx, &volumeClass); err != nil {
 		t.Fatal(err)
@@ -349,6 +364,7 @@ func TestVolumeClassRemoved(t *testing.T) {
 	volumeClass := adminapiv1.ClusterVolumeClass{
 		ObjectMeta:       metav1.ObjectMeta{Name: "acorn-test-custom"},
 		StorageClassName: getStorageClassName(t, storageClasses),
+		SupportedRegions: []string{apiv1.LocalRegion},
 	}
 	if err = kclient.Create(ctx, &volumeClass); err != nil {
 		t.Fatal(err)
@@ -420,6 +436,7 @@ func TestClusterVolumeClass(t *testing.T) {
 			Min:     v1.Quantity("1G"),
 			Max:     v1.Quantity("9G"),
 		},
+		SupportedRegions: []string{apiv1.LocalRegion},
 	}
 	if err = kclient.Create(ctx, &volumeClass); err != nil {
 		t.Fatal(err)
@@ -482,6 +499,7 @@ func TestClusterVolumeClassValuesInAcornfile(t *testing.T) {
 			Min:     v1.Quantity("1G"),
 			Max:     v1.Quantity("9G"),
 		},
+		SupportedRegions: []string{apiv1.LocalRegion},
 	}
 	if err = kclient.Create(ctx, &volumeClass); err != nil {
 		t.Fatal(err)
@@ -540,6 +558,7 @@ func TestProjectVolumeClass(t *testing.T) {
 			Min:     v1.Quantity("1G"),
 			Max:     v1.Quantity("3G"),
 		},
+		SupportedRegions: []string{apiv1.LocalRegion},
 	}
 	if err = kclient.Create(ctx, &volumeClass); err != nil {
 		t.Fatal(err)
@@ -604,7 +623,8 @@ func TestProjectVolumeClassDefaultSizeValidation(t *testing.T) {
 			Min:     v1.Quantity("1G"),
 			Max:     v1.Quantity("9G"),
 		},
-		Default: true,
+		Default:          true,
+		SupportedRegions: []string{apiv1.LocalRegion},
 	}
 	if err = kclient.Create(ctx, &volumeClass); err != nil {
 		t.Fatal(err)
@@ -715,6 +735,7 @@ func TestProjectVolumeClassValuesInAcornfile(t *testing.T) {
 			Min:     v1.Quantity("2G"),
 			Max:     v1.Quantity("6G"),
 		},
+		SupportedRegions: []string{apiv1.LocalRegion},
 	}
 	if err = kclient.Create(ctx, &volumeClass); err != nil {
 		t.Fatal(err)
@@ -786,7 +807,7 @@ func TestImageNameAnnotation(t *testing.T) {
 		}
 
 		_, digest, _ := strings.Cut(pod.Spec.Containers[0].Image, "sha256:")
-		return mapping["sha256:"+digest] == "public.ecr.aws/docker/library/nginx:latest"
+		return mapping["sha256:"+digest] == "ghcr.io/acorn-io/images-mirror/nginx:latest"
 	})
 }
 
@@ -895,6 +916,7 @@ func TestUsingComputeClasses(t *testing.T) {
 					Min: "512Mi",
 					Max: "1Gi",
 				},
+				SupportedRegions: []string{apiv1.LocalRegion},
 			},
 			expected: map[string]v1.Scheduling{"simple": {
 				Requirements: corev1.ResourceRequirements{
@@ -904,6 +926,12 @@ func TestUsingComputeClasses(t *testing.T) {
 						corev1.ResourceMemory: resource.MustParse("1Gi"),
 						corev1.ResourceCPU:    resource.MustParse("250m"),
 					},
+				},
+				Tolerations: []corev1.Toleration{
+					{
+						Key:      tolerations.WorkloadTolerationKey,
+						Operator: corev1.TolerationOpExists,
+					},
 				}},
 			},
 			waitFor: func(obj *apiv1.App) bool {
@@ -912,7 +940,7 @@ func TestUsingComputeClasses(t *testing.T) {
 			},
 		},
 		{
-			name:              "unrestricted default gets maximum",
+			name:              "unrestricted-default-gets-maximum",
 			testDataDirectory: "./testdata/computeclass",
 			computeClass: adminv1.ProjectComputeClassInstance{
 				ObjectMeta: metav1.ObjectMeta{
@@ -923,6 +951,7 @@ func TestUsingComputeClasses(t *testing.T) {
 				Memory: adminv1.ComputeClassMemory{
 					Max: "1Gi",
 				},
+				SupportedRegions: []string{apiv1.LocalRegion},
 			},
 			expected: map[string]v1.Scheduling{"simple": {
 				Requirements: corev1.ResourceRequirements{
@@ -932,15 +961,20 @@ func TestUsingComputeClasses(t *testing.T) {
 						corev1.ResourceMemory: resource.MustParse("1Gi"),
 						corev1.ResourceCPU:    resource.MustParse("250m"),
 					},
-				}},
-			},
+				},
+				Tolerations: []corev1.Toleration{
+					{
+						Key:      tolerations.WorkloadTolerationKey,
+						Operator: corev1.TolerationOpExists,
+					},
+				}}},
 			waitFor: func(obj *apiv1.App) bool {
 				return obj.Status.Condition(v1.AppInstanceConditionParsed).Success &&
 					obj.Status.Condition(v1.AppInstanceConditionScheduling).Success
 			},
 		},
 		{
-			name:              "with values",
+			name:              "with-values",
 			testDataDirectory: "./testdata/computeclass",
 			computeClass: adminv1.ProjectComputeClassInstance{
 				ObjectMeta: metav1.ObjectMeta{
@@ -955,6 +989,7 @@ func TestUsingComputeClasses(t *testing.T) {
 						"2Gi",
 					},
 				},
+				SupportedRegions: []string{apiv1.LocalRegion},
 			},
 			expected: map[string]v1.Scheduling{"simple": {
 				Requirements: corev1.ResourceRequirements{
@@ -964,6 +999,12 @@ func TestUsingComputeClasses(t *testing.T) {
 						corev1.ResourceMemory: resource.MustParse("1Gi"),
 						corev1.ResourceCPU:    resource.MustParse("250m"),
 					},
+				},
+				Tolerations: []corev1.Toleration{
+					{
+						Key:      tolerations.WorkloadTolerationKey,
+						Operator: corev1.TolerationOpExists,
+					},
 				}},
 			},
 			waitFor: func(obj *apiv1.App) bool {
@@ -986,6 +1027,7 @@ func TestUsingComputeClasses(t *testing.T) {
 					Max:     "1Gi",
 					Min:     "512Mi",
 				},
+				SupportedRegions: []string{apiv1.LocalRegion},
 			},
 			expected: map[string]v1.Scheduling{"simple": {
 				Requirements: corev1.ResourceRequirements{
@@ -995,6 +1037,12 @@ func TestUsingComputeClasses(t *testing.T) {
 						corev1.ResourceMemory: resource.MustParse("512Mi"),
 						corev1.ResourceCPU:    resource.MustParse("125m"),
 					},
+				},
+				Tolerations: []corev1.Toleration{
+					{
+						Key:      tolerations.WorkloadTolerationKey,
+						Operator: corev1.TolerationOpExists,
+					},
 				}},
 			},
 			waitFor: func(obj *apiv1.App) bool {
@@ -1003,7 +1051,26 @@ func TestUsingComputeClasses(t *testing.T) {
 			},
 		},
 		{
-			name:              "does not exist",
+			name:              "unsupported-region",
+			testDataDirectory: "./testdata/simple",
+			computeClass: adminv1.ProjectComputeClassInstance{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "acorn-test-custom",
+					Namespace: c.GetNamespace(),
+				},
+				Default:   true,
+				CPUScaler: 0.25,
+				Memory: adminv1.ComputeClassMemory{
+					Default: "512Mi",
+					Max:     "1Gi",
+					Min:     "512Mi",
+				},
+				SupportedRegions: []string{"non-local"},
+			},
+			fail: true,
+		},
+		{
+			name:              "does-not-exist",
 			noComputeClass:    true,
 			testDataDirectory: "./testdata/computeclass",
 			fail:              true,
@@ -1012,16 +1079,28 @@ func TestUsingComputeClasses(t *testing.T) {
 
 	for _, tt := range checks {
 		asClusterComputeClass := adminv1.ClusterComputeClassInstance(tt.computeClass)
-		for kind, computeClass := range map[string]runtimeclient.Object{"projectcomputeclass": &tt.computeClass, "clustercomputeclass": &asClusterComputeClass} {
-			t.Run(fmt.Sprintf("%v_%v", kind, tt.name), func(t *testing.T) {
+		// Perform the same test cases on both Project and Cluster ComputeClasses
+		for kind, computeClass := range map[string]crClient.Object{"projectcomputeclass": &tt.computeClass, "clustercomputeclass": &asClusterComputeClass} {
+			testcase := fmt.Sprintf("%v-%v", kind, tt.name)
+			t.Run(testcase, func(t *testing.T) {
 				if !tt.noComputeClass {
 					if err := kclient.Create(ctx, computeClass); err != nil {
 						t.Fatal(err)
 					}
+
+					// Clean-up and gurantee the computeclass doesn't exist after this test run
 					t.Cleanup(func() {
 						if err = kclient.Delete(context.Background(), computeClass); err != nil && !apierrors.IsNotFound(err) {
 							t.Fatal(err)
 						}
+						err := helper.EnsureDoesNotExist(ctx, func() (crClient.Object, error) {
+							lookingFor := computeClass
+							err := kclient.Get(ctx, router.Key(computeClass.GetNamespace(), computeClass.GetName()), lookingFor)
+							return lookingFor, err
+						})
+						if err != nil {
+							t.Fatal(err)
+						}
 					})
 				}
 
@@ -1032,170 +1111,136 @@ func TestUsingComputeClasses(t *testing.T) {
 					t.Fatal(err)
 				}
 
-				app, err := c.AppRun(ctx, image.ID, nil)
-				if err != nil {
-					if tt.fail {
-						return
+				// Assign a name for the test case so no collisions occur
+				app, err := c.AppRun(ctx, image.ID, &client.AppRunOptions{Name: testcase})
+				if err == nil && tt.fail {
+					t.Fatal("expected error, got nil")
+				} else if err != nil {
+					if !tt.fail {
+						t.Fatal(err)
 					}
-					t.Fatal(err)
 				}
 
-				app = helper.WaitForObject(t, helper.Watcher(t, c), new(apiv1.AppList), app, tt.waitFor)
-				assert.EqualValues(t, app.Status.Scheduling, tt.expected, "generated scheduling rules are incorrect")
+				// Clean-up and gurantee the app doesn't exist after this test run
+				if app != nil {
+					t.Cleanup(func() {
+						if err = kclient.Delete(context.Background(), app); err != nil && !apierrors.IsNotFound(err) {
+							t.Fatal(err)
+						}
+						err := helper.EnsureDoesNotExist(ctx, func() (crClient.Object, error) {
+							lookingFor := app
+							err := kclient.Get(ctx, router.Key(app.GetName(), app.GetNamespace()), lookingFor)
+							return lookingFor, err
+						})
+						if err != nil {
+							t.Fatal(err)
+						}
+					})
+				}
+
+				if tt.waitFor != nil {
+					app = helper.WaitForObject(t, helper.Watcher(t, c), new(apiv1.AppList), app, tt.waitFor)
+					assert.EqualValues(t, app.Status.Scheduling, tt.expected, "generated scheduling rules are incorrect")
+				}
 			})
 		}
 	}
 }
 
-func TestCreatingComputeClasses(t *testing.T) {
+func TestJobDelete(t *testing.T) {
 	helper.StartController(t)
-	cfg := helper.StartAPI(t)
-	ns := helper.TempNamespace(t, helper.MustReturn(kclient.Default))
-	kclient := helper.MustReturn(kclient.Default)
-	c, err := client.New(cfg, "", ns.Name)
+
+	ctx := helper.GetCTX(t)
+	c, _ := helper.ClientAndNamespace(t)
+
+	image, err := c.AcornImageBuild(ctx, "./testdata/jobfinalize/Acornfile", &client.AcornImageBuildOptions{
+		Cwd: "./testdata/jobfinalize",
+	})
 	if err != nil {
 		t.Fatal(err)
 	}
 
+	app, err := c.AppRun(ctx, image.ID, nil)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	app = helper.WaitForObject(t, helper.Watcher(t, c), new(apiv1.AppList), app, func(app *apiv1.App) bool {
+		return len(app.Finalizers) > 0 && app.Annotations[apply.AnnotationPrune] == "false"
+	})
+
+	app, err = c.AppDelete(ctx, app.Name)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_ = helper.EnsureDoesNotExist(ctx, func() (crClient.Object, error) {
+		return c.AppGet(ctx, app.Name)
+	})
+}
+
+func TestAppWithBadRegion(t *testing.T) {
+	helper.StartController(t)
+
 	ctx := helper.GetCTX(t)
+	c, _ := helper.ClientAndNamespace(t)
 
-	checks := []struct {
-		name      string
-		memory    adminv1.ComputeClassMemory
-		cpuScaler float64
-		fail      bool
-	}{
-		{
-			name: "valid-only-max",
-			memory: adminv1.ComputeClassMemory{
-				Max: "512Mi",
-			},
-			fail: false,
-		},
-		{
-			name: "valid-only-min",
-			memory: adminv1.ComputeClassMemory{
-				Min: "512Mi",
-			},
-			fail: false,
-		},
-		{
-			name: "valid-only-default",
-			memory: adminv1.ComputeClassMemory{
-				Default: "512Mi",
-			},
-			fail: false,
-		},
-		{
-			name:      "valid-values",
-			cpuScaler: 0.25,
-			memory: adminv1.ComputeClassMemory{
-				Default: "1Gi",
-				Values:  []string{"1Gi", "2Gi"},
-			},
-		},
-		{
-			name: "valid-empty",
-		},
-		{
-			name: "invalid-memory-default",
-			memory: adminv1.ComputeClassMemory{
-				Default: "invalid",
-			},
-			fail: true,
-		},
-		{
-			name: "invalid-memory-min",
-			memory: adminv1.ComputeClassMemory{
-				Min: "invalid",
-			},
-			fail: true,
-		},
-		{
-			name: "invalid-memory-max",
-			memory: adminv1.ComputeClassMemory{
-				Max: "invalid",
-			},
-			fail: true,
-		},
-		{
-			name: "invalid-memory-values",
-			memory: adminv1.ComputeClassMemory{
-				Values: []string{"invalid"},
-			},
-			fail: true,
-		},
-		{
-			name: "invalid-default-less-than-min",
-			memory: adminv1.ComputeClassMemory{
-				Default: "128Mi",
-				Min:     "512Mi",
-			},
-			fail: true,
-		},
-		{
-			name: "invalid-default-greater-than-max",
-			memory: adminv1.ComputeClassMemory{
-				Default: "1Gi",
-				Max:     "512Mi",
-			},
-			fail: true,
-		},
-		{
-			name: "invalid-min-max-swapped",
-			memory: adminv1.ComputeClassMemory{
-				Min: "1Gi",
-				Max: "512Mi",
-			},
-			fail: true,
-		},
-		{
-			name: "invalid-default-for-values",
-			memory: adminv1.ComputeClassMemory{
-				Default: "128Mi",
-				Values:  []string{"512Mi"},
-			},
-			fail: true,
-		},
-		{
-			name: "invalid-min-max-set-with-values",
-			memory: adminv1.ComputeClassMemory{
-				Min:    "512Mi",
-				Max:    "4Gi",
-				Values: []string{"2Gi", "3Gi"},
-			},
-			fail: true,
-		},
+	image, err := c.AcornImageBuild(ctx, "./testdata/simple/Acornfile", &client.AcornImageBuildOptions{
+		Cwd: "./testdata/simple",
+	})
+	if err != nil {
+		t.Fatal(err)
 	}
 
-	for _, tt := range checks {
-		t.Run(tt.name, func(t *testing.T) {
-			// Create a non-instanced ComputeClass to trigger Mink valdiation
-			computeClass := adminapiv1.ProjectComputeClass{
-				ObjectMeta: metav1.ObjectMeta{
-					GenerateName: "acorn-test-custom",
-					Namespace:    c.GetNamespace(),
-				},
-				CPUScaler: tt.cpuScaler,
-				Memory:    tt.memory,
-			}
+	_, err = c.AppRun(ctx, image.ID, &client.AppRunOptions{Region: "does-not-exist"})
+	if err == nil || !strings.Contains(err.Error(), "is not supported for project") {
+		t.Fatalf("expected an invalid region error, got %v", err)
+	}
+}
 
-			// TODO - dry run
-			err = kclient.Create(ctx, &computeClass)
-			if err != nil && !tt.fail {
-				t.Fatal("did not expect creation to fail:", err)
-			} else if err == nil {
-				if err := kclient.Delete(context.Background(), &computeClass); err != nil && !apierrors.IsNotFound(err) {
-					t.Fatal("failed to cleanup test:", err)
-				}
-				if tt.fail {
-					t.Fatal("expected an error to occur when creating an invalid ComputeClass but did not receive one")
-				}
-			}
+func TestAppWithBadDefaultRegion(t *testing.T) {
+	helper.StartController(t)
 
-		})
+	ctx := helper.GetCTX(t)
+	kclient := helper.MustReturn(kclient.Default)
+	c, ns := helper.ClientAndNamespace(t)
+
+	storageClasses := new(storagev1.StorageClassList)
+	err := kclient.List(ctx, storageClasses)
+	if err != nil || len(storageClasses.Items) == 0 {
+		t.Skip("No storage classes, so skipping TestAppWithBadDefaultRegion")
+		return
 	}
 
+	volumeClass := adminapiv1.ProjectVolumeClass{
+		ObjectMeta: metav1.ObjectMeta{
+			Namespace: ns.Name,
+			Name:      "acorn-test-custom",
+		},
+		StorageClassName: getStorageClassName(t, storageClasses),
+		Default:          true,
+		SupportedRegions: []string{"custom"},
+	}
+	if err = kclient.Create(ctx, &volumeClass); err != nil {
+		t.Fatal(err)
+	}
+	defer func() {
+		if err = kclient.Delete(context.Background(), &volumeClass); err != nil && !apierrors.IsNotFound(err) {
+			t.Fatal(err)
+		}
+	}()
+
+	image, err := c.AcornImageBuild(ctx, "./testdata/no-class-with-values/Acornfile", &client.AcornImageBuildOptions{
+		Cwd: "./testdata/no-class-with-values",
+	})
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = c.AppRun(ctx, image.ID, nil)
+	if err == nil || !strings.Contains(err.Error(), "is not a valid volume class") {
+		t.Fatalf("expected an invalid region error, got %v", err)
+	}
 }
 
 func getStorageClassName(t *testing.T, storageClasses *storagev1.StorageClassList) string {
@@ -1205,7 +1250,7 @@ func getStorageClassName(t *testing.T, storageClasses *storagev1.StorageClassLis
 	}
 
 	storageClassName := storageClasses.Items[0].Name
-	// Use local-=path if it exists
+	// Use local-path if it exists
 	for _, sc := range storageClasses.Items {
 		if sc.Name == "local-path" {
 			storageClassName = sc.Name
@@ -1214,3 +1259,379 @@ func getStorageClassName(t *testing.T, storageClasses *storagev1.StorageClassLis
 	}
 	return storageClassName
 }
+
+func TestCrossProjectNetworkConnection(t *testing.T) {
+	helper.StartController(t)
+
+	rc, err := restconfig.New(scheme.Scheme)
+	if err != nil {
+		t.Fatal("error while getting rest config:", err)
+	}
+	ctx := helper.GetCTX(t)
+	c, _ := helper.ClientAndNamespace(t)
+	kc := helper.MustReturn(kclient.Default)
+
+	cfg, err := config.Get(ctx, kc)
+	if err != nil {
+		t.Fatal(err)
+	} else if !*cfg.NetworkPolicies {
+		t.SkipNow() // skip this test because NetworkPolicies are not enabled
+	}
+
+	// create two separate projects in which to run two Nginx apps
+	proj1, err := c.ProjectCreate(ctx, "proj1", apiv1.LocalRegion, []string{apiv1.LocalRegion})
+	if err != nil {
+		t.Fatal("error while creating project:", err)
+	}
+	proj1Client, err := client.New(rc, proj1.Name, proj1.Status.Namespace)
+	if err != nil {
+		t.Fatal("error creating client for proj1:", err)
+	}
+
+	proj2, err := c.ProjectCreate(ctx, "proj2", apiv1.LocalRegion, []string{apiv1.LocalRegion})
+	if err != nil {
+		t.Fatal("error while creating project:", err)
+	}
+	proj2Client, err := client.New(rc, proj2.Name, proj2.Status.Namespace)
+	if err != nil {
+		t.Fatal("error creating client for proj2:", err)
+	}
+
+	t.Cleanup(func() {
+		// clean up projects
+		_, err := proj1Client.ProjectDelete(ctx, "proj1")
+		if err != nil {
+			t.Log("failed to delete project 'proj1':", err)
+		}
+		_, err = proj2Client.ProjectDelete(ctx, "proj2")
+		if err != nil {
+			t.Log("failed to delete project 'proj2':", err)
+		}
+	})
+
+	// create both apps, one in proj1 and the other in proj2
+	// app "foo" in proj1 does not publish any ports
+	// app "bar" in proj2 publishes port 80
+	fooImage, err := proj1Client.AcornImageBuild(ctx, "testdata/networkpolicy/Acornfile", nil)
+	if err != nil {
+		t.Fatal("error while building image:", err)
+	}
+	fooApp, err := proj1Client.AppRun(ctx, fooImage.ID, &client.AppRunOptions{
+		Name:            "foo",
+		TargetNamespace: proj1.Namespace,
+	})
+	if err != nil {
+		t.Fatal("error while running app:", err)
+	}
+
+	barImage, err := proj2Client.AcornImageBuild(ctx, "testdata/networkpolicy/publish.Acornfile", nil)
+	if err != nil {
+		t.Fatal("error while building image:", err)
+	}
+	barApp, err := proj2Client.AppRun(ctx, barImage.ID, &client.AppRunOptions{
+		Name:            "bar",
+		TargetNamespace: proj2.Namespace,
+	})
+	if err != nil {
+		t.Fatal("error while running app:", err)
+	}
+
+	// wait for both apps to be ready
+	helper.WaitForObject(t, helper.Watcher(t, proj1Client), &apiv1.AppList{}, fooApp, func(obj *apiv1.App) bool {
+		return obj.Status.Condition(v1.AppInstanceConditionReady).Success
+	})
+	helper.WaitForObject(t, helper.Watcher(t, proj2Client), &apiv1.AppList{}, barApp, func(obj *apiv1.App) bool {
+		return obj.Status.Condition(v1.AppInstanceConditionReady).Success
+	})
+
+	// determine pod IPs so we can test network connections
+	fooIP := getPodIPFromAppName(t, ctx, &kc, fooApp.Name, fooApp.Status.Namespace)
+	barIP := getPodIPFromAppName(t, ctx, &kc, barApp.Name, barApp.Status.Namespace)
+
+	// build an Acorn that just runs a job with the official curl container
+	curlImage1, err := proj1Client.AcornImageBuild(ctx, "testdata/networkpolicy/curl.Acornfile", nil)
+	if err != nil {
+		t.Fatal("error while building image:", err)
+	}
+	curlImage2, err := proj2Client.AcornImageBuild(ctx, "testdata/networkpolicy/curl.Acornfile", nil)
+	if err != nil {
+		t.Fatal("error while building image:", err)
+	}
+
+	checks := []struct {
+		name          string
+		client        client.Client
+		podIP         string
+		imageID       string
+		expectFailure bool
+	}{
+		{
+			name:          "curl-foo-proj1",
+			client:        proj1Client,
+			podIP:         fooIP,
+			imageID:       curlImage1.ID,
+			expectFailure: false,
+		},
+		{
+			name:    "curl-bar-proj1",
+			client:  proj1Client,
+			podIP:   barIP,
+			imageID: curlImage1.ID,
+			// even though bar has port 80 published, it should not be reachable from anything outside
+			// proj2 except for the ingress controller, so failure is expected here
+			expectFailure: true,
+		},
+		{
+			name:          "curl-foo-proj2",
+			client:        proj2Client,
+			podIP:         fooIP,
+			imageID:       curlImage2.ID,
+			expectFailure: true,
+		},
+		{
+			name:          "curl-bar-proj2",
+			client:        proj2Client,
+			podIP:         barIP,
+			imageID:       curlImage2.ID,
+			expectFailure: false,
+		},
+	}
+	for _, check := range checks {
+		t.Run(check.name, func(t *testing.T) {
+			// run curl to test the connection
+			app, err := check.client.AppRun(ctx, check.imageID, &client.AppRunOptions{
+				Name: check.name,
+				DeployArgs: map[string]any{
+					"address": check.podIP,
+				},
+			})
+			if err != nil {
+				t.Fatal("error while running app:", err)
+			}
+
+			appInstance := helper.WaitForObject(t, helper.Watcher(t, check.client), &apiv1.AppList{}, app, func(obj *apiv1.App) bool {
+				return obj.Status.AppStatus.Jobs["curl"].Ready || obj.Status.AppStatus.Jobs["curl"].ErrorCount > 0
+			})
+
+			if check.expectFailure {
+				assert.Equal(t, true, appInstance.Status.AppStatus.Jobs["curl"].ErrorCount > 0)
+			} else {
+				assert.Equal(t, true, appInstance.Status.AppStatus.Jobs["curl"].Ready)
+			}
+		})
+	}
+}
+
+func getPodIPFromAppName(t *testing.T, ctx context.Context, kc *crClient.WithWatch, appName, namespace string) string {
+	t.Helper()
+	selector, err := k8slabels.Parse(fmt.Sprintf("%s=%s", labels.AcornAppName, appName))
+	if err != nil {
+		t.Fatal("error creating k8s label selector:", err)
+	}
+
+	var podList corev1.PodList
+	podIP := ""
+	for podIP == "" {
+		err = (*kc).List(ctx, &podList, &kclient.ListOptions{
+			LabelSelector: selector,
+			Namespace:     namespace,
+		})
+		if err != nil {
+			t.Fatal("error listing pods:", err)
+		}
+		podIP = podList.Items[0].Status.PodIP
+	}
+
+	return podIP
+}
+
+func TestProjectUpdate(t *testing.T) {
+	helper.StartController(t)
+
+	rc, err := restconfig.New(scheme.Scheme)
+	if err != nil {
+		t.Fatal("error while getting rest config:", err)
+	}
+	ctx := helper.GetCTX(t)
+	c, _ := helper.ClientAndNamespace(t)
+	projectName := uuid.New().String()[:8]
+	proj1, err := c.ProjectCreate(ctx, projectName, apiv1.LocalRegion, []string{apiv1.LocalRegion})
+	if err != nil {
+		t.Fatal("error while creating project:", err)
+	}
+	proj1Client, err := client.New(rc, proj1.Name, proj1.Status.Namespace)
+	if err != nil {
+		t.Fatal("error creating client for project:", err)
+	}
+	watchClient, err := proj1Client.GetClient()
+	if err != nil {
+		t.Fatal("error creating watch client for project:", err)
+	}
+	t.Cleanup(func() {
+		// clean up projects
+		_, err := proj1Client.ProjectDelete(ctx, projectName)
+		if err != nil {
+			t.Logf("failed to delete project '%s': %s", projectName, err)
+		}
+	})
+	var latestProject *apiv1.Project
+	var updatedProj *apiv1.Project
+	latestProject, err = watcher.New[*apiv1.Project](watchClient).ByObject(ctx, proj1, func(latestProject *apiv1.Project) (bool, error) {
+		if latestProject != nil && latestProject.Status.Namespace != "" {
+			return true, nil
+		}
+		return false, nil
+	})
+	if err != nil {
+		t.Fatal("error while waiting for project to be created:", err)
+	}
+	// update default
+	for i := 0; i < 10; i++ {
+		updatedProj, err = proj1Client.ProjectUpdate(ctx, latestProject, "new-default", []string{apiv1.LocalRegion})
+		if err == nil {
+			break
+		}
+		if !apierrors.IsConflict(err) {
+			t.Fatal("error while updating project:", err)
+		}
+		latestProject, err = proj1Client.ProjectGet(ctx, projectName)
+		if err != nil {
+			t.Fatal("error while getting project:", err)
+		}
+	}
+
+	assert.Equal(t, updatedProj.Spec.DefaultRegion, "new-default")
+	assert.Equal(t, updatedProj.Spec.SupportedRegions, []string{apiv1.LocalRegion, "new-default"})
+
+	// swap default from new-default to local
+	for i := 0; i < 10; i++ {
+		updatedProj, err = proj1Client.ProjectGet(ctx, projectName)
+		if err != nil {
+			t.Fatal("error while getting project:", err)
+		}
+		updatedProj, err = proj1Client.ProjectUpdate(ctx, updatedProj, apiv1.LocalRegion, nil)
+		if err == nil {
+			break
+		}
+		if !apierrors.IsConflict(err) {
+			t.Fatal("error while updating project:", err)
+		}
+	}
+	assert.Equal(t, updatedProj.Spec.DefaultRegion, apiv1.LocalRegion)
+	assert.Equal(t, updatedProj.Spec.SupportedRegions, []string{apiv1.LocalRegion, "new-default"})
+
+	// remove new-default region
+	for i := 0; i < 10; i++ {
+		updatedProj, err = proj1Client.ProjectGet(ctx, projectName)
+		if err != nil {
+			t.Fatal("error while getting project:", err)
+		}
+		updatedProj, err = proj1Client.ProjectUpdate(ctx, updatedProj, "", []string{apiv1.LocalRegion})
+		if err == nil {
+			break
+		}
+		if !strings.Contains(err.Error(), "please apply your changes to the latest version and try again") {
+			t.Fatal("error while updating project:", err)
+		}
+	}
+	assert.Equal(t, updatedProj.Spec.DefaultRegion, apiv1.LocalRegion)
+	assert.Equal(t, updatedProj.Spec.SupportedRegions, []string{apiv1.LocalRegion})
+
+	// set supported regions
+	for i := 0; i < 10; i++ {
+		updatedProj, err = proj1Client.ProjectGet(ctx, projectName)
+		if err != nil {
+			t.Fatal("error while getting project:", err)
+		}
+		updatedProj, err = proj1Client.ProjectUpdate(ctx, updatedProj, "", []string{apiv1.LocalRegion, "local3", "local2"})
+		if err == nil {
+			break
+		}
+		if !apierrors.IsConflict(err) {
+			t.Fatal("error while updating project:", err)
+		}
+	}
+	assert.Equal(t, updatedProj.Spec.DefaultRegion, apiv1.LocalRegion)
+	assert.Equal(t, updatedProj.Spec.SupportedRegions, []string{apiv1.LocalRegion, "local3", "local2"})
+}
+
+func TestEnforcedQuota(t *testing.T) {
+	ctx := helper.GetCTX(t)
+
+	helper.StartController(t)
+	restConfig, err := restconfig.New(scheme.Scheme)
+	if err != nil {
+		t.Fatal("error while getting rest config:", err)
+	}
+	// Create a project.
+	kclient := helper.MustReturn(kclient.Default)
+	project := helper.TempProject(t, kclient)
+
+	// Create a client for the project.
+	c, err := client.New(restConfig, project.Name, project.Name)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	// Annotate the project to enforce quota.
+	helper.WaitForObject(t, helper.Watcher(t, c), &corev1.NamespaceList{}, project, func(obj *corev1.Namespace) bool {
+		if obj.Annotations == nil {
+			obj.Annotations = make(map[string]string)
+		}
+		obj.Annotations[labels.ProjectEnforcedQuotaAnnotation] = "true"
+		return kclient.Update(ctx, obj) == nil
+	})
+
+	// Run a simple app.
+	image, err := c.AcornImageBuild(ctx, "./testdata/simple/Acornfile", &client.AcornImageBuildOptions{
+		Cwd: "./testdata/volume",
+	})
+	if err != nil {
+		t.Fatal(err)
+	}
+	app, err := c.AppRun(ctx, image.ID, nil)
+	if err != nil {
+		t.Fatal(err)
+	}
+	t.Cleanup(func() {
+		_, err = c.AppDelete(ctx, app.Name)
+		if err != nil {
+			t.Fatal(err)
+		}
+	})
+
+	// Wait for the app to set the AppInstanceQuotaAllocated condition to be transitioning and for the namespace
+	// to be ready.
+	helper.WaitForObject(t, helper.Watcher(t, c), &apiv1.AppList{}, app, func(obj *apiv1.App) bool {
+		return obj.Status.Condition(v1.AppInstanceConditionQuotaAllocated).Transitioning
+	})
+
+	// Grab the app's QuotaRequest and check that it has the appropriate values set.
+	quotaRequest := &adminv1.QuotaRequestInstance{}
+	err = kclient.Get(ctx, router.Key(app.Namespace, app.Name), quotaRequest)
+	if err != nil {
+		t.Fatal(err)
+	}
+	assert.Equal(t, quotaRequest.Spec.Resources.Containers, 1)
+
+	// Update the status of the QuotaRequest to communicate readiness.
+	quotaRequest.Status = adminv1.QuotaRequestInstanceStatus{
+		ObservedGeneration: quotaRequest.Generation,
+		Conditions: []v1.Condition{{
+			ObservedGeneration: quotaRequest.Generation,
+			Type:               adminv1.QuotaRequestCondition,
+			Status:             metav1.ConditionTrue,
+			Success:            true,
+		}},
+		AllocatedResources: quotaRequest.Spec.Resources,
+	}
+	err = kclient.Status().Update(ctx, quotaRequest)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	// Wait for the app to set the AppInstanceQuotaAllocated condition to success.
+	helper.WaitForObject(t, helper.Watcher(t, c), &apiv1.AppList{}, app, func(obj *apiv1.App) bool {
+		return obj.Status.Condition(v1.AppInstanceConditionQuotaAllocated).Success
+	})
+}
