@@ -15,16 +15,55 @@ type ContainerImageBuilderSpec struct {
 	Sidecars map[string]ContainerImageBuilderSpec `json:"sidecars,omitempty"`
 }
 
+func (in *ContainerImageBuilderSpec) Normalize() *ContainerImageBuilderSpec {
+	out := *in
+	if out.Image != "" {
+		out.Build = nil
+	}
+	if len(in.Sidecars) > 0 {
+		out.Sidecars = map[string]ContainerImageBuilderSpec{}
+		for k, v := range in.Sidecars {
+			out.Sidecars[k] = *v.Normalize()
+		}
+	}
+	return &out
+}
+
 type ImageBuilderSpec struct {
-	Image string `json:"image,omitempty"`
-	Build *Build `json:"build,omitempty"`
+	Image          string      `json:"image,omitempty"`
+	ContainerBuild *Build      `json:"containerBuild,omitempty"`
+	AcornBuild     *AcornBuild `json:"acornBuild,omitempty"`
+}
+
+func (in *ImageBuilderSpec) Normalize() *ImageBuilderSpec {
+	out := *in
+	if out.Image != "" {
+		out.ContainerBuild = nil
+		out.AcornBuild = nil
+	}
+	return &out
+}
+
+type AcornBuilderSpec struct {
+	AutoUpgrade bool        `json:"autoUpgrade,omitempty"`
+	Image       string      `json:"image,omitempty"`
+	Build       *AcornBuild `json:"build,omitempty"`
+}
+
+func (in *AcornBuilderSpec) Normalize() *AcornBuilderSpec {
+	out := *in
+	if out.Image != "" {
+		out.Build = nil
+	}
+	return &out
 }
 
 type BuilderSpec struct {
-	Platforms  []Platform                           `json:"platforms,omitempty"`
+	Services   map[string]AcornBuilderSpec          `json:"services,omitempty"`
 	Containers map[string]ContainerImageBuilderSpec `json:"containers,omitempty"`
 	Jobs       map[string]ContainerImageBuilderSpec `json:"jobs,omitempty"`
 	Images     map[string]ImageBuilderSpec          `json:"images,omitempty"`
+	Acorns     map[string]AcornBuilderSpec          `json:"acorns,omitempty"`
 }
 
 type ParamSpec struct {
@@ -57,9 +96,9 @@ type AcornImageBuildInstance struct {
 type AcornImageBuildInstanceSpec struct {
 	BuilderName string     `json:"builderName,omitempty" wrangler:"required"`
 	Acornfile   string     `json:"acornfile,omitempty"`
+	Profiles    []string   `json:"profiles,omitempty"`
 	Platforms   []Platform `json:"platforms,omitempty"`
 	Args        GenericMap `json:"args,omitempty"`
-	Profiles    []string   `json:"profiles,omitempty"`
 	VCS         VCS        `json:"vcs,omitempty"`
 }
 
@@ -71,12 +110,25 @@ type AcornImageBuildInstanceStatus struct {
 	AppImage           AppImage    `json:"appImage,omitempty"`
 	Conditions         []Condition `json:"conditions,omitempty"`
 	BuildError         string      `json:"buildError,omitempty"`
+	Region             string      `json:"region,omitempty"`
 }
 
 func (in *AcornImageBuildInstance) Conditions() *[]Condition {
 	return &in.Status.Conditions
 }
 
+func (in *AcornImageBuildInstance) HasRegion(region string) bool {
+	return in.Status.Region == region
+}
+
+func (in *AcornImageBuildInstance) GetRegion() string {
+	return in.Status.Region
+}
+
+func (in *AcornImageBuildInstance) SetDefaultRegion(region string) {
+	in.Status.Region = region
+}
+
 // +k8s:deepcopy-gen:interfaces=k8s.io/apimachinery/pkg/runtime.Object
 
 type AcornImageBuildInstanceList struct {
@@ -101,6 +153,19 @@ type BuilderInstanceStatus struct {
 	Endpoint           string `json:"endpoint,omitempty"`
 	PublicKey          string `json:"publicKey,omitempty"`
 	ServiceName        string `json:"serviceName,omitempty"`
+	Region             string `json:"region,omitempty"`
+}
+
+func (b *BuilderInstance) HasRegion(region string) bool {
+	return b.Status.Region == region
+}
+
+func (b *BuilderInstance) GetRegion() string {
+	return b.Status.Region
+}
+
+func (b *BuilderInstance) SetDefaultRegion(region string) {
+	b.Status.Region = region
 }
 
 // +k8s:deepcopy-gen:interfaces=k8s.io/apimachinery/pkg/runtime.Object
