@@ -11,8 +11,8 @@ import (
 	"strconv"
 	"strings"
 
+	"github.com/acorn-io/aml"
 	"github.com/google/shlex"
-	rbacv1 "k8s.io/api/rbac/v1"
 	"k8s.io/apimachinery/pkg/api/resource"
 )
 
@@ -160,6 +160,34 @@ func (in *Quantity) UnmarshalJSON(data []byte) error {
 	return nil
 }
 
+func (in *MemoryMap) UnmarshalJSON(data []byte) error {
+	if isObject(data) {
+		return json.Unmarshal(data, (*map[string]*int64)(in))
+	}
+	i, err := aml.ParseInt(string(data))
+	if err != nil {
+		return err
+	}
+	*in = MemoryMap{
+		"": &i,
+	}
+	return nil
+}
+
+func (in *ServiceBindings) UnmarshalJSON(data []byte) error {
+	if isArray(data) {
+		return json.Unmarshal(data, (*[]ServiceBinding)(in))
+	}
+
+	var serviceBinding ServiceBinding
+	if err := json.Unmarshal(data, &serviceBinding); err != nil {
+		return err
+	}
+
+	*in = append(*in, serviceBinding)
+	return nil
+}
+
 func (in *ServiceBinding) UnmarshalJSON(data []byte) error {
 	if !isString(data) {
 		type serviceBinding ServiceBinding
@@ -179,6 +207,20 @@ func (in *ServiceBinding) UnmarshalJSON(data []byte) error {
 	return nil
 }
 
+func (in *SecretBindings) UnmarshalJSON(data []byte) error {
+	if isArray(data) {
+		return json.Unmarshal(data, (*[]SecretBinding)(in))
+	}
+
+	var secretBinding SecretBinding
+	if err := json.Unmarshal(data, &secretBinding); err != nil {
+		return err
+	}
+
+	*in = append(*in, secretBinding)
+	return nil
+}
+
 func (in *SecretBinding) UnmarshalJSON(data []byte) error {
 	if !isString(data) {
 		type secretBinding SecretBinding
@@ -197,6 +239,20 @@ func (in *SecretBinding) UnmarshalJSON(data []byte) error {
 	return nil
 }
 
+func (in *VolumeBindings) UnmarshalJSON(data []byte) error {
+	if isArray(data) {
+		return json.Unmarshal(data, (*[]VolumeBinding)(in))
+	}
+
+	var volumeBinding VolumeBinding
+	if err := json.Unmarshal(data, &volumeBinding); err != nil {
+		return err
+	}
+
+	*in = append(*in, volumeBinding)
+	return nil
+}
+
 func (in *VolumeBinding) UnmarshalJSON(data []byte) error {
 	if !isString(data) {
 		type volumeBinding VolumeBinding
@@ -217,6 +273,9 @@ func (in *VolumeBinding) UnmarshalJSON(data []byte) error {
 
 func impliedSecretsForContainer(app *AppSpec, container Container) {
 	for _, env := range container.Environment {
+		if strings.Contains(env.Secret.Name, ".") {
+			continue
+		}
 		if _, ok := app.Secrets[env.Secret.Name]; env.Secret.Name != "" && !ok {
 			app.Secrets[env.Secret.Name] = Secret{
 				Type: "opaque",
@@ -303,26 +362,10 @@ func impliedVolumesForContainer(app *AppSpec, containerName, sideCarName string,
 
 func checkForDuplicateNames(in *AppSpec) error {
 	names := map[string]string{}
-	for name, c := range in.Containers {
-		for _, port := range c.Ports {
-			if port.ServiceName != "" && port.ServiceName != name {
-				if err := addName(names, port.ServiceName, "port"); err != nil {
-					return err
-				}
-			}
-		}
+	for name := range in.Containers {
 		if err := addName(names, name, "container"); err != nil {
 			return err
 		}
-		for _, sidecar := range c.Sidecars {
-			for _, port := range sidecar.Ports {
-				if port.ServiceName != "" && port.ServiceName != name {
-					if err := addName(names, port.ServiceName, "port"); err != nil {
-						return err
-					}
-				}
-			}
-		}
 	}
 	for name := range in.Jobs {
 		if err := addName(names, name, "job"); err != nil {
@@ -334,6 +377,21 @@ func checkForDuplicateNames(in *AppSpec) error {
 			return err
 		}
 	}
+	for name := range in.Acorns {
+		if err := addName(names, name, "acorn"); err != nil {
+			return err
+		}
+	}
+	for name := range in.Services {
+		if err := addName(names, name, "service"); err != nil {
+			return err
+		}
+	}
+	for name := range in.Images {
+		if err := addName(names, name, "image"); err != nil {
+			return err
+		}
+	}
 
 	return nil
 }
@@ -346,6 +404,24 @@ func addImpliedResources(in *AppSpec) error {
 		in.Secrets = map[string]Secret{}
 	}
 
+	for _, a := range in.Acorns {
+		for _, volumeBinding := range a.Volumes {
+			if _, ok := in.Volumes[volumeBinding.Volume]; !ok {
+				in.Volumes[volumeBinding.Volume] = VolumeRequest{
+					Size:        volumeBinding.Size,
+					AccessModes: volumeBinding.AccessModes,
+				}
+			}
+		}
+		for _, secretBinding := range a.Secrets {
+			if _, ok := in.Secrets[secretBinding.Secret]; !ok {
+				in.Secrets[secretBinding.Secret] = Secret{
+					Type: "opaque",
+				}
+			}
+		}
+	}
+
 	for containerName, c := range in.Containers {
 		impliedSecretsForContainer(in, c)
 		if err := impliedVolumesForContainer(in, containerName, "", c); err != nil {
@@ -418,6 +494,50 @@ func (in *ContainerImageBuilderSpec) UnmarshalJSON(data []byte) error {
 	return nil
 }
 
+func (in *PortBinding) UnmarshalJSON(data []byte) error {
+	defer func() {
+		if in.ZZ_ServiceName != "" && in.Hostname == "" {
+			in.Hostname = in.ZZ_ServiceName
+		}
+	}()
+
+	if isObject(data) {
+		type portBinding PortBinding
+		return json.Unmarshal(data, (*portBinding)(in))
+	} else if !isString(data) {
+		i, err := strconv.Atoi(string(data))
+		if err != nil {
+			return fmt.Errorf("parsing [%s]: %w", data, err)
+		}
+		in.TargetPort = int32(i)
+		return nil
+	}
+
+	s, err := parseString(data)
+	if err != nil {
+		return err
+	}
+	portBinding, err := ParsePortBindings([]string{s})
+	if err != nil {
+		return err
+	}
+	*in = portBinding[0]
+	return nil
+}
+
+func (in *PortBindings) UnmarshalJSON(data []byte) error {
+	if isArray(data) {
+		return json.Unmarshal(data, (*[]PortBinding)(in))
+	}
+
+	var portBinding PortBinding
+	if err := json.Unmarshal(data, &portBinding); err != nil {
+		return err
+	}
+	*in = append(*in, portBinding)
+	return nil
+}
+
 func (in *AccessModes) UnmarshalJSON(data []byte) error {
 	if !isString(data) {
 		return json.Unmarshal(data, (*[]AccessMode)(in))
@@ -430,6 +550,31 @@ func (in *AccessModes) UnmarshalJSON(data []byte) error {
 	return nil
 }
 
+type acornAliases struct {
+	Env NameValues `json:"env,omitempty"`
+	Mem MemoryMap  `json:"mem,omitempty"`
+}
+
+func (a acornAliases) SetService(dst Service) Service {
+	if len(a.Env) > 0 {
+		dst.Environment = append(dst.Environment, a.Env...)
+	}
+	if len(a.Mem) > 0 {
+		dst.Memory = a.Mem
+	}
+	return dst
+}
+
+func (a acornAliases) SetAcorn(dst Acorn) Acorn {
+	if len(a.Env) > 0 {
+		dst.Environment = append(dst.Environment, a.Env...)
+	}
+	if len(a.Mem) > 0 {
+		dst.Memory = a.Mem
+	}
+	return dst
+}
+
 type containerAliases struct {
 	Cmd                 CommandSlice           `json:"cmd,omitempty"`
 	Env                 EnvVars                `json:"env,omitempty"`
@@ -501,6 +646,23 @@ func adjustBuildForContextDirs(c Container) *Build {
 	return build
 }
 
+func (in *Acorn) UnmarshalJSON(data []byte) error {
+	var a Acorn
+	type acorn Acorn
+	if err := json.Unmarshal(data, (*acorn)(&a)); err != nil {
+		return err
+	}
+
+	var alias acornAliases
+	if err := json.Unmarshal(data, &alias); err != nil {
+		return err
+	}
+
+	a = alias.SetAcorn(a)
+	*in = a
+	return nil
+}
+
 func (in *Container) UnmarshalJSON(data []byte) error {
 	var c Container
 	type container Container
@@ -525,35 +687,46 @@ func (in *Container) UnmarshalJSON(data []byte) error {
 	return nil
 }
 
-func (in *PolicyRule) UnmarshalJSON(data []byte) error {
-	if !isString(data) {
-		return json.Unmarshal(data, (*rbacv1.PolicyRule)(in))
-	}
-
-	s, err := parseString(data)
-	if err != nil {
-		return err
-	}
-	read := strings.HasPrefix(s, "read ")
-	if read {
-		s = strings.TrimPrefix(s, "read ")
-	}
-
-	resource, apiGroup, _ := strings.Cut(s, ".")
-	in.Resources = []string{resource}
-	in.APIGroups = []string{apiGroup}
-	in.Verbs = DefaultVerbs
-	if read {
-		in.Verbs = ReadVerbs
-	}
-
-	return nil
+type policyRuleAliases struct {
+	Verb         string   `json:"verb,omitempty"`
+	APIGroup     string   `json:"apiGroup,omitempty"`
+	Resource     string   `json:"resource,omitempty"`
+	ResourceName string   `json:"resourceName,omitempty"`
+	Scope        string   `json:"scope,omitempty"`
+	Namespaces   []string `json:"namespaces,omitempty"`
 }
 
-func (in *ClusterPolicyRule) UnmarshalJSON(data []byte) error {
+func (in *PolicyRule) UnmarshalJSON(data []byte) error {
 	if !isString(data) {
-		type clusterPolicyRule ClusterPolicyRule
-		return json.Unmarshal(data, (*clusterPolicyRule)(in))
+		type policyRule PolicyRule
+		if err := json.Unmarshal(data, (*policyRule)(in)); err != nil {
+			return err
+		}
+		alias := &policyRuleAliases{}
+		if err := json.Unmarshal(data, alias); err != nil {
+			return err
+		}
+		if alias.Verb != "" {
+			in.Verbs = append(in.Verbs, alias.Verb)
+		}
+		if alias.APIGroup != "" {
+			in.APIGroups = append(in.APIGroups, alias.APIGroup)
+		}
+		if alias.Resource != "" {
+			in.Resources = append(in.Resources, alias.Resource)
+		}
+		if alias.ResourceName != "" {
+			in.ResourceNames = append(in.ResourceNames, alias.ResourceName)
+		}
+		if alias.Scope != "" {
+			in.Scopes = append(in.Scopes, alias.Scope)
+		}
+		if len(alias.Namespaces) > 0 {
+			for _, ns := range alias.Namespaces {
+				in.Scopes = append(in.Scopes, "namespace:"+ns)
+			}
+		}
+		return nil
 	}
 
 	s, err := parseString(data)
@@ -566,6 +739,9 @@ func (in *ClusterPolicyRule) UnmarshalJSON(data []byte) error {
 	}
 
 	resource, apiGroup, _ := strings.Cut(s, ".")
+	if apiGroup == "" {
+		apiGroup = "api.acorn.io"
+	}
 	in.Resources = []string{resource}
 	in.APIGroups = []string{apiGroup}
 	in.Verbs = DefaultVerbs
@@ -636,13 +812,18 @@ func (in *Ports) UnmarshalJSON(data []byte) error {
 		if err := json.Unmarshal(data, &ports); err != nil {
 			return err
 		}
-		for _, port := range ports["expose"] {
-			port.Expose = true
-			*in = append(*in, port)
-		}
+		*in = append(*in, ports["expose"]...)
 		*in = append(*in, ports["internal"]...)
+		for i := range *in {
+			(*in)[i].Publish = false
+		}
 		for _, port := range ports["publish"] {
 			port.Publish = true
+			port.Dev = false
+			*in = append(*in, port)
+		}
+		for _, port := range ports["dev"] {
+			port.Dev = true
 			*in = append(*in, port)
 		}
 		return nil
@@ -956,6 +1137,20 @@ func (in *CommandSlice) UnmarshalJSON(data []byte) error {
 	return json.Unmarshal(data, (*commandSlice)(in))
 }
 
+func (in *AcornBuild) UnmarshalJSON(data []byte) error {
+	if isString(data) {
+		s, err := parseString(data)
+		if err != nil {
+			return err
+		}
+		in.Context = s
+		in.Acornfile = filepath.Join(s, "Acornfile")
+		return nil
+	}
+	type acornBuild AcornBuild
+	return json.Unmarshal(data, (*acornBuild)(in))
+}
+
 func (in *Build) UnmarshalJSON(data []byte) error {
 	if isString(data) {
 		s, err := parseString(data)
@@ -1179,3 +1374,20 @@ func ParseNameValues(fillEnv bool, s ...string) (result []NameValue) {
 	}
 	return result
 }
+
+func (in *Service) UnmarshalJSON(data []byte) error {
+	var a Service
+	type acorn Service
+	if err := json.Unmarshal(data, (*acorn)(&a)); err != nil {
+		return err
+	}
+
+	var alias acornAliases
+	if err := json.Unmarshal(data, &alias); err != nil {
+		return err
+	}
+
+	a = alias.SetService(a)
+	*in = a
+	return nil
+}
