@@ -4,6 +4,7 @@ import (
 	"archive/tar"
 	"bytes"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
 	"path/filepath"
@@ -23,6 +24,10 @@ const (
 	BuildDataFile = "build.json"
 )
 
+var (
+	ErrInvalidInput = errors.New("invalid input")
+)
+
 type AppDefinition struct {
 	data         []byte
 	imageDatas   []v1.ImagesData
