@@ -145,7 +145,7 @@ containers: {
         build: {}
       }
       right: {
-        image: "public.ecr.aws/docker/library/nginx:latest"
+        image: "ghcr.io/acorn-io/images-mirror/nginx:latest"
         dirs: "/var/tmp": "./foo/bar"
       }
     }
@@ -163,20 +163,41 @@ containers: {
 }
 
 images: {
-  file: {
-    build: "sub/dir1"
+  ifile: {
+    acornBuild: "sub/dir1"
   }
-  none: {
+  inone: {
     image: "done"
   }
-  full: {
-    build: {
+  ifull: {
+    containerBuild: {
       context: "sub/dir2"	
       dockerfile: "sub/dir3/Dockerfile"
       target: "other"
     }
   }
 }
+
+acorns: {
+  afile: {
+    build: "sub/dir1"
+  }
+  anone: {
+    image: "done"
+  }
+  afull: {
+    build: {
+      context: "sub/dir2"
+      acornfile: "sub/dir3/Acornfile"
+      buildArgs: {
+        key: "value"
+        key2: {
+          key3: "value3"
+		}
+	  }
+    }
+  }
+}
 `))
 	if err != nil {
 		t.Fatal(err)
@@ -201,21 +222,32 @@ images: {
 	assert.Equal(t, "Dockerfile", buildSpec.Containers["file"].Sidecars["left"].Build.Dockerfile)
 	assert.Equal(t, ".", buildSpec.Containers["file"].Sidecars["left"].Build.Context)
 
-	assert.Equal(t, "public.ecr.aws/docker/library/nginx:latest", buildSpec.Containers["file"].Sidecars["right"].Image)
+	assert.Equal(t, "ghcr.io/acorn-io/images-mirror/nginx:latest", buildSpec.Containers["file"].Sidecars["right"].Image)
 	assert.Equal(t, "Dockerfile", buildSpec.Containers["file"].Sidecars["right"].Build.Dockerfile)
-	assert.Equal(t, "public.ecr.aws/docker/library/nginx:latest", buildSpec.Containers["file"].Sidecars["right"].Build.BaseImage)
+	assert.Equal(t, "ghcr.io/acorn-io/images-mirror/nginx:latest", buildSpec.Containers["file"].Sidecars["right"].Build.BaseImage)
 	assert.Equal(t, ".", buildSpec.Containers["file"].Sidecars["right"].Build.Context)
 	assert.Equal(t, "./foo/bar", buildSpec.Containers["file"].Sidecars["right"].Build.ContextDirs["/var/tmp"])
 
 	assert.Len(t, buildSpec.Images, 3)
-	assert.Equal(t, "", buildSpec.Images["file"].Image)
-	assert.Equal(t, "sub/dir1", buildSpec.Images["file"].Build.Context)
-	assert.Equal(t, filepath.Join("sub", "dir1", "Dockerfile"), buildSpec.Images["file"].Build.Dockerfile)
+	assert.Equal(t, "", buildSpec.Images["ifile"].Image)
+	assert.Equal(t, "sub/dir1", buildSpec.Images["ifile"].AcornBuild.Context)
+	assert.Equal(t, filepath.Join("sub", "dir1", "Acornfile"), buildSpec.Images["ifile"].AcornBuild.Acornfile)
 	assert.Equal(t, "", buildSpec.Images["full"].Image)
-	assert.Equal(t, "sub/dir2", buildSpec.Images["full"].Build.Context)
-	assert.Equal(t, "sub/dir3/Dockerfile", buildSpec.Images["full"].Build.Dockerfile)
-	assert.Equal(t, "other", buildSpec.Images["full"].Build.Target)
-	assert.Equal(t, "done", buildSpec.Images["none"].Image)
+	assert.Equal(t, "sub/dir2", buildSpec.Images["ifull"].ContainerBuild.Context)
+	assert.Equal(t, "sub/dir3/Dockerfile", buildSpec.Images["ifull"].ContainerBuild.Dockerfile)
+	assert.Equal(t, "other", buildSpec.Images["ifull"].ContainerBuild.Target)
+	assert.Equal(t, "done", buildSpec.Images["inone"].Image)
+
+	assert.Len(t, buildSpec.Acorns, 3)
+	assert.Equal(t, "", buildSpec.Acorns["afile"].Image)
+	assert.Equal(t, "sub/dir1", buildSpec.Acorns["afile"].Build.Context)
+	assert.Equal(t, filepath.Join("sub", "dir1", "Acornfile"), buildSpec.Acorns["afile"].Build.Acornfile)
+	assert.Equal(t, "", buildSpec.Acorns["afull"].Image)
+	assert.Equal(t, "sub/dir2", buildSpec.Acorns["afull"].Build.Context)
+	assert.Equal(t, "sub/dir3/Acornfile", buildSpec.Acorns["afull"].Build.Acornfile)
+	assert.Equal(t, "value", buildSpec.Acorns["afull"].Build.BuildArgs["key"])
+	assert.Equal(t, map[string]any{"key3": "value3"}, buildSpec.Acorns["afull"].Build.BuildArgs["key2"])
+	assert.Equal(t, "done", buildSpec.Acorns["anone"].Image)
 }
 
 func TestWatchFiles(t *testing.T) {
@@ -245,17 +277,17 @@ containers: {
 }
 
 images: {
-  file: {
-    build: "sub/dir2"
+  ifile: {
+    acornBuild: "sub/dir2"
   }
-  none: {
+  inone: {
     image: "done"
   }
-  two: {
-    build: {}
+  itwo: {
+    acornBuild: {}
   }
-  full: {
-    build: {
+  ifull: {
+    containerBuild: {
       dockerfile: "sub/dir3/bockerfile"
     }
   }
@@ -271,11 +303,15 @@ images: {
 	}
 
 	assert.Equal(t, []string{
+		filepath.Join("root-path", ".dockerignore"),
+		filepath.Join("root-path", "Acornfile"),
 		filepath.Join("root-path", "Dockerfile"),
+		filepath.Join("root-path", "asdf", ".dockerignore"),
 		filepath.Join("root-path", "asdf", "dockerfile"),
 		filepath.Join("root-path", "dockerfile.sidecar"),
+		filepath.Join("root-path", "sub", "dir1", ".dockerignore"),
 		filepath.Join("root-path", "sub", "dir1", "Dockerfile"),
-		filepath.Join("root-path", "sub", "dir2", "Dockerfile"),
+		filepath.Join("root-path", "sub", "dir2", "Acornfile"),
 		filepath.Join("root-path", "sub", "dir3", "bockerfile"),
 	}, files)
 }
@@ -540,7 +576,6 @@ containers: {
 			},
 		},
 	}, appSpec.Containers["a"].Environment)
-
 }
 
 func TestWorkdir(t *testing.T) {
@@ -742,69 +777,55 @@ containers: {
 		t.Fatal(err)
 	}
 
-	assert.Equal(t, false, appSpec.Containers["s"].Ports[0].Expose)
 	assert.Equal(t, int32(80), appSpec.Containers["s"].Ports[0].Port)
 	assert.Equal(t, int32(80), appSpec.Containers["s"].Ports[0].TargetPort)
 	assert.Equal(t, appSpec.Containers["s"].Ports[0].Protocol, v1.Protocol(""))
 
-	assert.Equal(t, false, appSpec.Containers["s"].Ports[1].Expose)
 	assert.Equal(t, int32(80), appSpec.Containers["s"].Ports[1].Port)
 	assert.Equal(t, int32(81), appSpec.Containers["s"].Ports[1].TargetPort)
 	assert.Equal(t, appSpec.Containers["s"].Ports[1].Protocol, v1.Protocol(""))
 
-	assert.Equal(t, false, appSpec.Containers["s"].Ports[2].Expose)
-	assert.Equal(t, int32(80), appSpec.Containers["s"].Ports[2].Port)
+	assert.Equal(t, int32(0), appSpec.Containers["s"].Ports[2].Port)
 	assert.Equal(t, int32(80), appSpec.Containers["s"].Ports[2].TargetPort)
 	assert.Equal(t, appSpec.Containers["s"].Ports[2].Protocol, v1.ProtocolHTTP)
 
-	assert.Equal(t, false, appSpec.Containers["s"].Ports[3].Expose)
 	assert.Equal(t, int32(80), appSpec.Containers["s"].Ports[3].Port)
 	assert.Equal(t, int32(81), appSpec.Containers["s"].Ports[3].TargetPort)
 	assert.Equal(t, appSpec.Containers["s"].Ports[3].Protocol, v1.ProtocolHTTP)
 
-	assert.Equal(t, true, appSpec.Containers["s"].Sidecars["left"].Ports[0].Expose)
-	assert.Equal(t, int32(80), appSpec.Containers["s"].Sidecars["left"].Ports[0].Port)
+	assert.Equal(t, int32(0), appSpec.Containers["s"].Sidecars["left"].Ports[0].Port)
 	assert.Equal(t, int32(80), appSpec.Containers["s"].Sidecars["left"].Ports[0].TargetPort)
 	assert.Equal(t, appSpec.Containers["s"].Sidecars["left"].Ports[0].Protocol, v1.Protocol(""))
 
-	assert.Equal(t, true, appSpec.Containers["s"].Sidecars["left"].Ports[1].Expose)
 	assert.Equal(t, int32(80), appSpec.Containers["s"].Sidecars["left"].Ports[1].Port)
 	assert.Equal(t, int32(81), appSpec.Containers["s"].Sidecars["left"].Ports[1].TargetPort)
 	assert.Equal(t, appSpec.Containers["s"].Sidecars["left"].Ports[1].Protocol, v1.ProtocolTCP)
 
-	assert.Equal(t, true, appSpec.Containers["s"].Sidecars["left"].Ports[2].Expose)
-	assert.Equal(t, int32(80), appSpec.Containers["s"].Sidecars["left"].Ports[2].Port)
+	assert.Equal(t, int32(0), appSpec.Containers["s"].Sidecars["left"].Ports[2].Port)
 	assert.Equal(t, int32(80), appSpec.Containers["s"].Sidecars["left"].Ports[2].TargetPort)
 	assert.Equal(t, appSpec.Containers["s"].Sidecars["left"].Ports[2].Protocol, v1.ProtocolHTTP)
 
-	assert.Equal(t, true, appSpec.Containers["s"].Sidecars["left"].Ports[3].Expose)
 	assert.Equal(t, int32(80), appSpec.Containers["s"].Sidecars["left"].Ports[3].Port)
 	assert.Equal(t, int32(81), appSpec.Containers["s"].Sidecars["left"].Ports[3].TargetPort)
 	assert.Equal(t, appSpec.Containers["s"].Sidecars["left"].Ports[3].Protocol, v1.ProtocolHTTP)
 
-	assert.Equal(t, true, appSpec.Containers["s"].Sidecars["left2"].Ports[0].Expose)
 	assert.Equal(t, false, appSpec.Containers["s"].Sidecars["left2"].Ports[0].Publish)
-	assert.Equal(t, int32(81), appSpec.Containers["s"].Sidecars["left2"].Ports[0].Port)
+	assert.Equal(t, int32(0), appSpec.Containers["s"].Sidecars["left2"].Ports[0].Port)
 	assert.Equal(t, int32(81), appSpec.Containers["s"].Sidecars["left2"].Ports[0].TargetPort)
 	assert.Equal(t, appSpec.Containers["s"].Sidecars["left2"].Ports[0].Protocol, v1.ProtocolHTTP)
 	assert.Equal(t, true, appSpec.Containers["s"].Sidecars["left2"].Ports[1].Publish)
-	assert.Equal(t, false, appSpec.Containers["s"].Sidecars["left2"].Ports[1].Expose)
 	assert.Equal(t, int32(80), appSpec.Containers["s"].Sidecars["left2"].Ports[1].Port)
 	assert.Equal(t, int32(80), appSpec.Containers["s"].Sidecars["left2"].Ports[1].TargetPort)
 	assert.Equal(t, appSpec.Containers["s"].Sidecars["left2"].Ports[1].Protocol, v1.Protocol(""))
 
-	assert.Equal(t, false, appSpec.Containers["s"].Sidecars["right"].Ports[0].Expose)
-	assert.Equal(t, int32(80), appSpec.Containers["s"].Sidecars["right"].Ports[0].Port)
+	assert.Equal(t, int32(0), appSpec.Containers["s"].Sidecars["right"].Ports[0].Port)
 	assert.Equal(t, appSpec.Containers["s"].Sidecars["right"].Ports[0].Protocol, v1.Protocol(""))
 
-	assert.Equal(t, false, appSpec.Containers["s"].Sidecars["right2"].Ports[0].Expose)
 	assert.Equal(t, int32(80), appSpec.Containers["s"].Sidecars["right2"].Ports[0].Port)
 	assert.Equal(t, appSpec.Containers["s"].Sidecars["right2"].Ports[0].Protocol, v1.Protocol(""))
 
-	assert.Equal(t, false, appSpec.Containers["s"].Sidecars["right3"].Ports[0].Expose)
 	assert.Equal(t, int32(12), appSpec.Containers["s"].Sidecars["right3"].Ports[0].Port)
 	assert.Equal(t, appSpec.Containers["s"].Sidecars["right3"].Ports[0].Protocol, v1.Protocol(""))
-	assert.Equal(t, false, appSpec.Containers["s"].Sidecars["right3"].Ports[1].Expose)
 	assert.Equal(t, int32(80), appSpec.Containers["s"].Sidecars["right3"].Ports[1].Port)
 	assert.Equal(t, appSpec.Containers["s"].Sidecars["right3"].Ports[1].Protocol, v1.Protocol(""))
 }
@@ -859,7 +880,7 @@ containers: {
 	assert.Equal(t, int32(81), appSpec.Containers["s"].Ports[1].TargetPort)
 	assert.Equal(t, appSpec.Containers["s"].Ports[1].Protocol, v1.Protocol(""))
 
-	assert.Equal(t, int32(80), appSpec.Containers["s"].Ports[2].Port)
+	assert.Equal(t, int32(0), appSpec.Containers["s"].Ports[2].Port)
 	assert.Equal(t, int32(80), appSpec.Containers["s"].Ports[2].TargetPort)
 	assert.Equal(t, appSpec.Containers["s"].Ports[2].Protocol, v1.ProtocolHTTP)
 
@@ -867,7 +888,7 @@ containers: {
 	assert.Equal(t, int32(81), appSpec.Containers["s"].Ports[3].TargetPort)
 	assert.Equal(t, appSpec.Containers["s"].Ports[3].Protocol, v1.ProtocolHTTP)
 
-	assert.Equal(t, int32(80), appSpec.Containers["s"].Sidecars["left"].Ports[0].Port)
+	assert.Equal(t, int32(0), appSpec.Containers["s"].Sidecars["left"].Ports[0].Port)
 	assert.Equal(t, int32(80), appSpec.Containers["s"].Sidecars["left"].Ports[0].TargetPort)
 	assert.Equal(t, appSpec.Containers["s"].Sidecars["left"].Ports[0].Protocol, v1.Protocol(""))
 
@@ -875,7 +896,7 @@ containers: {
 	assert.Equal(t, int32(81), appSpec.Containers["s"].Sidecars["left"].Ports[1].TargetPort)
 	assert.Equal(t, appSpec.Containers["s"].Sidecars["left"].Ports[1].Protocol, v1.ProtocolTCP)
 
-	assert.Equal(t, int32(80), appSpec.Containers["s"].Sidecars["left"].Ports[2].Port)
+	assert.Equal(t, int32(0), appSpec.Containers["s"].Sidecars["left"].Ports[2].Port)
 	assert.Equal(t, int32(80), appSpec.Containers["s"].Sidecars["left"].Ports[2].TargetPort)
 	assert.Equal(t, appSpec.Containers["s"].Sidecars["left"].Ports[2].Protocol, v1.ProtocolHTTP)
 
@@ -883,7 +904,7 @@ containers: {
 	assert.Equal(t, int32(81), appSpec.Containers["s"].Sidecars["left"].Ports[3].TargetPort)
 	assert.Equal(t, appSpec.Containers["s"].Sidecars["left"].Ports[3].Protocol, v1.ProtocolHTTP)
 
-	assert.Equal(t, int32(80), appSpec.Containers["s"].Sidecars["right"].Ports[0].Port)
+	assert.Equal(t, int32(0), appSpec.Containers["s"].Sidecars["right"].Ports[0].Port)
 	assert.Equal(t, appSpec.Containers["s"].Sidecars["right"].Ports[0].Protocol, v1.Protocol(""))
 
 	assert.Equal(t, int32(80), appSpec.Containers["s"].Sidecars["right2"].Ports[0].Port)
@@ -965,10 +986,10 @@ containers: {
 	}
 }
 images: {
-	build: {
-      build: {}
+	ibuild: {
+      acornBuild: {}
     }
-    image: {
+    iimage: {
       image: "images-image-image"
 	}
 }`))
@@ -982,7 +1003,8 @@ images: {
 	}
 
 	assert.Equal(t, &v1.BuilderSpec{
-		Jobs: map[string]v1.ContainerImageBuilderSpec{},
+		Services: map[string]v1.AcornBuilderSpec{},
+		Jobs:     map[string]v1.ContainerImageBuilderSpec{},
 		Containers: map[string]v1.ContainerImageBuilderSpec{
 			"image": {
 				Image: "image-image",
@@ -1056,17 +1078,18 @@ images: {
 			},
 		},
 		Images: map[string]v1.ImageBuilderSpec{
-			"build": {
-				Build: &v1.Build{
-					BuildArgs:  map[string]string{},
-					Context:    ".",
-					Dockerfile: "Dockerfile",
+			"ibuild": {
+				AcornBuild: &v1.AcornBuild{
+					BuildArgs: v1.GenericMap{},
+					Context:   ".",
+					Acornfile: "Acornfile",
 				},
 			},
-			"image": {
+			"iimage": {
 				Image: "images-image-image",
 			},
 		},
+		Acorns: map[string]v1.AcornBuilderSpec{},
 	}, buildSpec)
 
 	app := appImage.WithImageData(v1.ImagesData{
@@ -1105,10 +1128,10 @@ images: {
 			},
 		},
 		Images: map[string]v1.ImageData{
-			"build": {
+			"ibuild": {
 				Image: "images-build-image",
 			},
-			"image": {
+			"iimage": {
 				Image: "images-image-image",
 			},
 		},
@@ -1355,7 +1378,6 @@ secrets: {
 	assert.Equal(t, v1.Secret{
 		Type: "opaque",
 	}, appSpec.Secrets["opt"])
-
 }
 
 func TestImageDataOverride(t *testing.T) {
@@ -1567,6 +1589,363 @@ containers: foo: build: buildArgs: one: args.foo
 	assert.Equal(t, "two", buildSpec.Containers["foo"].Build.BuildArgs["one"])
 }
 
+func TestAcornFirstForm(t *testing.T) {
+	acornCue := `
+acorns: first: {
+	labels: [
+		{
+			resourceType: "container"
+			resourceName: "containerName"
+			key: "keyname"
+			value: "value"
+		},
+		{
+			resourceType: "acorn"
+			resourceName: "containerName2"
+			key: "keyname2"
+			value: "value2"
+		}
+	]
+	annotations: [
+		{
+			resourceType: "container"
+			resourceName: "containerName"
+			key: "keyname"
+			value: "value"
+		},
+		{
+			resourceType: "acorn"
+			resourceName: "containerName2"
+			key: "keyname2"
+			value: "value2"
+		}
+	]
+	image: "acornimage"
+	build: "./dir"
+	publish: 80
+	volumes: "b"
+	secrets: "a:b"
+	links: "b:c"
+	autoUpgrade: true
+	autoUpgradeInterval: "20s"
+	notifyUpgrade: true
+	mem: 1Gi
+	env: [
+		"a=b",
+		"c=d"
+	]
+	deployArgs: foo: 12
+	profiles: ["abc", "def"]
+}
+`
+
+	def, err := NewAppDefinition([]byte(acornCue))
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	appSpec, err := def.AppSpec()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	acorn := appSpec.Acorns["first"]
+
+	assert.Equal(t, v1.ScopedLabel{
+		ResourceType: "container",
+		ResourceName: "containerName",
+		Key:          "keyname",
+		Value:        "value",
+	}, acorn.Labels[0])
+	assert.Equal(t, v1.ScopedLabel{
+		ResourceType: "acorn",
+		ResourceName: "containerName2",
+		Key:          "keyname2",
+		Value:        "value2",
+	}, acorn.Labels[1])
+	assert.Equal(t, v1.ScopedLabel{
+		ResourceType: "container",
+		ResourceName: "containerName",
+		Key:          "keyname",
+		Value:        "value",
+	}, acorn.Annotations[0])
+	assert.Equal(t, v1.ScopedLabel{
+		ResourceType: "acorn",
+		ResourceName: "containerName2",
+		Key:          "keyname2",
+		Value:        "value2",
+	}, acorn.Annotations[1])
+	assert.Equal(t, "acornimage", acorn.Image)
+	assert.Equal(t, &v1.AcornBuild{
+		Context:   "./dir",
+		Acornfile: "dir/Acornfile",
+	}, acorn.Build)
+	assert.Equal(t, v1.VolumeBinding{
+		Volume: "",
+		Target: "b",
+	}, acorn.Volumes[0])
+	assert.Equal(t, v1.SecretBinding{
+		Secret: "a",
+		Target: "b",
+	}, acorn.Secrets[0])
+	assert.Equal(t, v1.ServiceBinding{
+		Service: "b",
+		Target:  "c",
+	}, acorn.Links[0])
+	assert.True(t, *acorn.AutoUpgrade)
+	assert.True(t, *acorn.NotifyUpgrade)
+	assert.Equal(t, "20s", acorn.AutoUpgradeInterval)
+	assert.Equal(t, v1.MemoryMap{
+		"": &[]int64{1 << 30}[0],
+	}, acorn.Memory)
+	assert.Equal(t, v1.NameValue{
+		Name:  "a",
+		Value: "b",
+	}, acorn.Environment[0])
+	assert.Equal(t, v1.NameValue{
+		Name:  "c",
+		Value: "d",
+	}, acorn.Environment[1])
+	assert.Equal(t, v1.GenericMap{
+		"foo": int64(12),
+	}, acorn.DeployArgs)
+	assert.Equal(t, []string{"abc", "def"}, acorn.Profiles)
+}
+
+func TestAcornSecondForm(t *testing.T) {
+	acornCue := `
+acorns: first: {
+	labels: {
+		"containers:containername:abc.io/keyname": "value",
+	}
+	annotations: {
+		"acorn:containername2:abc.io/keyname2": "value2",
+	}
+	build: {
+		context: "abc"
+		acornfile: "other/Acornfile"
+		buildArgs: a: "b"
+	}
+	publish: "example.com:1234/http"
+	volumes: [{
+		target: "def"
+	}]
+	secrets: [{
+		secret: "abc"
+		target: "def"
+	}]
+	links: [{
+		service: "abc"
+		target: "def"
+	}]
+	mem: {
+		"foo": 1Gi
+	}
+	env: {
+		"X": "v"
+	}
+}
+`
+
+	def, err := NewAppDefinition([]byte(acornCue))
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	appSpec, err := def.AppSpec()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	acorn := appSpec.Acorns["first"]
+
+	assert.Equal(t, v1.ScopedLabel{
+		ResourceType: "container",
+		ResourceName: "containername",
+		Key:          "abc.io/keyname",
+		Value:        "value",
+	}, acorn.Labels[0])
+	assert.Equal(t, v1.ScopedLabel{
+		ResourceType: "acorn",
+		ResourceName: "containername2",
+		Key:          "abc.io/keyname2",
+		Value:        "value2",
+	}, acorn.Annotations[0])
+	assert.Equal(t, &v1.AcornBuild{
+		Context:   "abc",
+		Acornfile: "other/Acornfile",
+		BuildArgs: v1.GenericMap{
+			"a": "b",
+		},
+	}, acorn.Build)
+	assert.Equal(t, v1.PortBinding{
+		Hostname:   "example.com",
+		TargetPort: 1234,
+		Protocol:   v1.ProtocolHTTP,
+	}, acorn.Publish[0])
+	assert.Equal(t, v1.VolumeBinding{
+		Volume: "",
+		Target: "def",
+	}, acorn.Volumes[0])
+	assert.Equal(t, v1.SecretBinding{
+		Secret: "abc",
+		Target: "def",
+	}, acorn.Secrets[0])
+	assert.Equal(t, v1.ServiceBinding{
+		Service: "abc",
+		Target:  "def",
+	}, acorn.Links[0])
+	assert.Equal(t, v1.MemoryMap{
+		"foo": &[]int64{1 << 30}[0],
+	}, acorn.Memory)
+	assert.Equal(t, v1.NameValues{
+		{
+			Name:  "X",
+			Value: "v",
+		},
+	}, acorn.Environment)
+}
+
+func TestAcornThirdForm(t *testing.T) {
+	acornCue := `
+acorns: first: {
+	publish: [{
+		hostname: "example.com"
+		targetPort: 80
+		targetServiceName: "asdf"
+	}, 80, "example.com:123"]
+}
+`
+
+	def, err := NewAppDefinition([]byte(acornCue))
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	appSpec, err := def.AppSpec()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	acorn := appSpec.Acorns["first"]
+
+	assert.Equal(t, v1.PortBinding{
+		Hostname:          "example.com",
+		Port:              80,
+		TargetPort:        80,
+		TargetServiceName: "asdf",
+	}, acorn.Publish[0])
+	assert.Equal(t, v1.PortBinding{
+		TargetPort: 80,
+	}, acorn.Publish[1])
+	assert.Equal(t, v1.PortBinding{
+		Hostname:   "example.com",
+		TargetPort: 123,
+		Protocol:   v1.ProtocolHTTP,
+	}, acorn.Publish[2])
+}
+
+func TestService(t *testing.T) {
+	acornCue := `
+services: first: {
+	default: true
+	external: "externalname"
+	address: "some.address"
+	ports: "80"
+	container: "foo"
+	secrets: ["sfoo", "sbar"]
+	data: foo: hi: "bye"
+}
+
+services: acorn: {
+	default: true
+	labels: foo: "bar"
+	image: "foo"
+	build: "./dir"
+	secrets: "foo:bar"
+	env: foo: "bar"
+	autoUpgrade: true
+	autoUpgradeInterval:   "5m"
+	notifyUpgrade:         true
+	mem:      44G
+	env: foo: "bar"
+	serviceArgs: key: "value"
+}
+
+services: job: {
+	default: true
+	labels: foo: "bar"
+	generated: job: "foo"
+}
+`
+
+	def, err := NewAppDefinition([]byte(acornCue))
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	appSpec, err := def.AppSpec()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	svc := appSpec.Services["first"]
+
+	assert.Equal(t, true, svc.Default)
+	assert.Equal(t, "externalname", svc.External)
+	assert.Equal(t, "some.address", svc.Address)
+	assert.Equal(t, v1.PortDef{
+		TargetPort: 80,
+	}, svc.Ports[0])
+	assert.Equal(t, "foo", svc.Container)
+	assert.Equal(t, v1.SecretBindings{
+		{
+			Secret: "sfoo",
+			Target: "sfoo",
+		},
+		{
+			Secret: "sbar",
+			Target: "sbar",
+		},
+	}, svc.Secrets)
+	assert.Equal(t, v1.GenericMap{
+		"foo": map[string]any{
+			"hi": "bye",
+		},
+	}, svc.Data)
+
+	acorn := appSpec.Services["acorn"]
+
+	assert.Equal(t, true, acorn.Default)
+	assert.Equal(t, "foo", acorn.Image)
+	assert.Equal(t, "./dir", acorn.Build.Context)
+	assert.Equal(t, "foo", acorn.Labels[0].Key)
+	assert.Equal(t, "bar", acorn.Labels[0].Value)
+	assert.Equal(t, "foo", acorn.Image)
+	assert.Equal(t, "./dir", acorn.Build.Context)
+	assert.Equal(t, v1.SecretBindings{
+		{
+			Secret: "foo",
+			Target: "bar",
+		},
+	}, acorn.Secrets)
+	assert.Equal(t, true, *acorn.AutoUpgrade)
+	assert.Equal(t, "5m", acorn.AutoUpgradeInterval)
+	assert.Equal(t, true, *acorn.NotifyUpgrade)
+	assert.Equal(t, int64(44000000000), *acorn.Memory[""])
+	assert.Equal(t, "foo", acorn.Environment[0].Name)
+	assert.Equal(t, "bar", acorn.Environment[0].Value)
+	assert.Equal(t, v1.GenericMap{
+		"key": "value",
+	}, acorn.ServiceArgs)
+
+	job := appSpec.Services["job"]
+	assert.Equal(t, true, job.Default)
+	assert.Equal(t, "foo", job.Generated.Job)
+	assert.Equal(t, "foo", job.Labels[0].Key)
+	assert.Equal(t, "bar", job.Labels[0].Value)
+}
+
 func TestCronJob(t *testing.T) {
 	acornCue := `
 jobs: foo: {
@@ -1587,7 +1966,7 @@ jobs: foo: {
 	assert.Equal(t, "daily", appSpec.Jobs["foo"].Schedule)
 }
 
-func TestAliasNotMatchContainer(t *testing.T) {
+func TestInvalidPublishHostname(t *testing.T) {
 	acornCue := `
 containers: foo: {
   ports: "foo2:80"
@@ -1598,17 +1977,38 @@ containers: foo2: image: "image"
 
 	_, err := NewAppDefinition([]byte(acornCue))
 	assert.NotNil(t, err)
-	assert.Contains(t, err.Error(), "duplicate name [foo2] used by [")
+	assert.Contains(t, err.Error(), "[foo2] is not a valid hostname to publish, missing \".\"")
+}
+
+func TestLink(t *testing.T) {
+	acornCue := `
+acorns: one: links: ["two:three", "one"]
+`
+
+	def, err := NewAppDefinition([]byte(acornCue))
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	appSpec, err := def.AppSpec()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	assert.Equal(t, "two", appSpec.Acorns["one"].Links[0].Service)
+	assert.Equal(t, "three", appSpec.Acorns["one"].Links[0].Target)
+	assert.Equal(t, "one", appSpec.Acorns["one"].Links[1].Service)
+	assert.Equal(t, "one", appSpec.Acorns["one"].Links[1].Target)
 }
 
 func TestAlias(t *testing.T) {
 	acornCue := `
 containers: foo: {
-  ports: "foo2:80"
+  ports: "foo2.local:80"
   image: "image"
 }
 containers: foo3: {
-  ports: "foo4:80"
+  ports: "foo4.local:80"
   image: "image"
 }
 `
@@ -1623,8 +2023,8 @@ containers: foo3: {
 		t.Fatal(err)
 	}
 
-	assert.Equal(t, "foo2", appSpec.Containers["foo"].Ports[0].ServiceName)
-	assert.Equal(t, "foo4", appSpec.Containers["foo3"].Ports[0].ServiceName)
+	assert.Equal(t, "foo2.local", appSpec.Containers["foo"].Ports[0].Hostname)
+	assert.Equal(t, "foo4.local", appSpec.Containers["foo3"].Ports[0].Hostname)
 }
 
 func TestProbes(t *testing.T) {
@@ -1781,16 +2181,7 @@ localData: permissions: {
       resources: ["resources"]
       resourceNames: ["names"]
       nonResourceURLs: ["foo"]
-    }
-  ]
-  clusterRules: [
-    {
-      verbs: ["verb"]
-      apiGroups: ["groups"]
-      resources: ["resources"]
-      resourceNames: ["names"]
-      nonResourceURLs: ["foo"]
-	  namespaces: ["a", "b"]
+      scopes: ["foo"]
     }
   ]
 }
@@ -1816,28 +2207,14 @@ containers: cont: {
 	assert.Equal(t, "resources", appSpec.Containers["cont"].Permissions.Rules[0].Resources[0])
 	assert.Equal(t, "names", appSpec.Containers["cont"].Permissions.Rules[0].ResourceNames[0])
 	assert.Equal(t, "foo", appSpec.Containers["cont"].Permissions.Rules[0].NonResourceURLs[0])
-
-	assert.Equal(t, "verb", appSpec.Containers["cont"].Permissions.ClusterRules[0].Verbs[0])
-	assert.Equal(t, "groups", appSpec.Containers["cont"].Permissions.ClusterRules[0].APIGroups[0])
-	assert.Equal(t, "resources", appSpec.Containers["cont"].Permissions.ClusterRules[0].Resources[0])
-	assert.Equal(t, "names", appSpec.Containers["cont"].Permissions.ClusterRules[0].ResourceNames[0])
-	assert.Equal(t, "a", appSpec.Containers["cont"].Permissions.ClusterRules[0].Namespaces[0])
-	assert.Equal(t, "b", appSpec.Containers["cont"].Permissions.ClusterRules[0].Namespaces[1])
-	assert.Equal(t, "foo", appSpec.Containers["cont"].Permissions.ClusterRules[0].NonResourceURLs[0])
+	assert.Equal(t, "foo", appSpec.Containers["cont"].Permissions.Rules[0].Scopes[0])
 
 	assert.Equal(t, "verb", appSpec.Containers["cont"].Sidecars["side"].Permissions.Rules[0].Verbs[0])
 	assert.Equal(t, "groups", appSpec.Containers["cont"].Sidecars["side"].Permissions.Rules[0].APIGroups[0])
 	assert.Equal(t, "resources", appSpec.Containers["cont"].Sidecars["side"].Permissions.Rules[0].Resources[0])
 	assert.Equal(t, "names", appSpec.Containers["cont"].Sidecars["side"].Permissions.Rules[0].ResourceNames[0])
 	assert.Equal(t, "foo", appSpec.Containers["cont"].Sidecars["side"].Permissions.Rules[0].NonResourceURLs[0])
-
-	assert.Equal(t, "verb", appSpec.Containers["cont"].Sidecars["side"].Permissions.ClusterRules[0].Verbs[0])
-	assert.Equal(t, "groups", appSpec.Containers["cont"].Sidecars["side"].Permissions.ClusterRules[0].APIGroups[0])
-	assert.Equal(t, "resources", appSpec.Containers["cont"].Sidecars["side"].Permissions.ClusterRules[0].Resources[0])
-	assert.Equal(t, "names", appSpec.Containers["cont"].Sidecars["side"].Permissions.ClusterRules[0].ResourceNames[0])
-	assert.Equal(t, "a", appSpec.Containers["cont"].Sidecars["side"].Permissions.ClusterRules[0].Namespaces[0])
-	assert.Equal(t, "b", appSpec.Containers["cont"].Sidecars["side"].Permissions.ClusterRules[0].Namespaces[1])
-	assert.Equal(t, "foo", appSpec.Containers["cont"].Sidecars["side"].Permissions.ClusterRules[0].NonResourceURLs[0])
+	assert.Equal(t, "foo", appSpec.Containers["cont"].Sidecars["side"].Permissions.Rules[0].Scopes[0])
 }
 
 func TestNoImport(t *testing.T) {
@@ -2119,6 +2496,35 @@ containers: a: env: a: ""
 	assert.Equal(t, "", appSpec.Containers["a"].Environment[0].Value)
 }
 
+func TestEmptyAcornEnv(t *testing.T) {
+	data := `
+acorns: slice: env: ["a=b", "c=d"]
+acorns: m: env: a: "b"
+acorns: m: env: c: "d"
+`
+	appDef, err := NewAppDefinition([]byte(data))
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	appSpec, err := appDef.AppSpec()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	assert.Len(t, appSpec.Acorns["slice"].Environment, 2)
+	assert.Equal(t, "a", appSpec.Acorns["slice"].Environment[0].Name)
+	assert.Equal(t, "b", appSpec.Acorns["slice"].Environment[0].Value)
+	assert.Equal(t, "c", appSpec.Acorns["slice"].Environment[1].Name)
+	assert.Equal(t, "d", appSpec.Acorns["slice"].Environment[1].Value)
+
+	assert.Len(t, appSpec.Acorns["m"].Environment, 2)
+	assert.Equal(t, "a", appSpec.Acorns["m"].Environment[0].Name)
+	assert.Equal(t, "b", appSpec.Acorns["m"].Environment[0].Value)
+	assert.Equal(t, "c", appSpec.Acorns["m"].Environment[1].Name)
+	assert.Equal(t, "d", appSpec.Acorns["m"].Environment[1].Value)
+}
+
 func TestTemplateSecretCustomNames(t *testing.T) {
 	data := `
 containers: test: {
@@ -2188,7 +2594,7 @@ containers: test: {
 	assert.Equal(t, "api.group", appSpec.Containers["test"].Permissions.Rules[0].APIGroups[0])
 	assert.Equal(t, v1.DefaultVerbs, appSpec.Containers["test"].Permissions.Rules[0].Verbs)
 	assert.Equal(t, "secrets", appSpec.Containers["test"].Permissions.Rules[1].Resources[0])
-	assert.Equal(t, "", appSpec.Containers["test"].Permissions.Rules[1].APIGroups[0])
+	assert.Equal(t, "api.acorn.io", appSpec.Containers["test"].Permissions.Rules[1].APIGroups[0])
 	assert.Equal(t, []string{"get", "list", "watch"}, appSpec.Containers["test"].Permissions.Rules[1].Verbs)
 }
 
@@ -2237,8 +2643,69 @@ containers: default: image: "foo"
 }
 
 func TestNestedScopedLabels(t *testing.T) {
+	// labels and annotations on a acorn are both unmarshalled into a ScopedLabels struct, which is just a slice
+	// Similar to ports, in the Acornfile you can define them using an object syntax or short-form string syntax.
+	// We're testing that they get unmarshalled properly
+	appImage, err := NewAppDefinition([]byte(`acorns: {
+	s: {
+		labels: [
+			{
+				resourceType: "",
+				resourceName: "",
+				key: "globalkey",
+				value: "x0"
+			},
+			{
+				resourceType: "containers",
+				resourceName: "",
+				key: "conskey",
+				value: "x1"
+			},
+			{
+				resourceType: "containers",
+				resourceName: "confoo",
+				key: "fookey",
+				value: "x2"
+			},
+			{
+				resourceType: "",
+				resourceName: "objbar",
+				key: "acorn.io/barkey",
+				value: "x3"
+			}
+		]
+		annotations: {
+			globalkey: "v0"
+			"containers:conskey": "v1"
+			"containers:confoo:acorn.io/fookey": "v2"
+			"conbar:barkey": "v3"
+		}
+		image: ""
+	}}`))
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	appSpec, err := appImage.AppSpec()
+	if err != nil {
+		errors.Print(os.Stderr, err, nil)
+		t.Fatal(err)
+	}
+
+	// Asserting that labels defined using the struct/object form are properly unmarshalled
+	assert.Equal(t, v1.ScopedLabel{ResourceType: "", ResourceName: "", Key: "globalkey", Value: "x0"}, appSpec.Acorns["s"].Labels[0])
+	assert.Equal(t, v1.ScopedLabel{ResourceType: "container", ResourceName: "", Key: "conskey", Value: "x1"}, appSpec.Acorns["s"].Labels[1])
+	assert.Equal(t, v1.ScopedLabel{ResourceType: "container", ResourceName: "confoo", Key: "fookey", Value: "x2"}, appSpec.Acorns["s"].Labels[2])
+	assert.Equal(t, v1.ScopedLabel{ResourceType: "", ResourceName: "objbar", Key: "acorn.io/barkey", Value: "x3"}, appSpec.Acorns["s"].Labels[3])
+
+	// Asserting that annotations defined using the shorthand string form are properly unmarshalled
+	assert.Equal(t, v1.ScopedLabel{ResourceType: "", ResourceName: "", Key: "globalkey", Value: "v0"}, appSpec.Acorns["s"].Annotations[0])
+	assert.Equal(t, v1.ScopedLabel{ResourceType: "", ResourceName: "conbar", Key: "barkey", Value: "v3"}, appSpec.Acorns["s"].Annotations[1])
+	assert.Equal(t, v1.ScopedLabel{ResourceType: "container", ResourceName: "", Key: "conskey", Value: "v1"}, appSpec.Acorns["s"].Annotations[2])
+	assert.Equal(t, v1.ScopedLabel{ResourceType: "container", ResourceName: "confoo", Key: "acorn.io/fookey", Value: "v2"}, appSpec.Acorns["s"].Annotations[3])
+
 	// Expect error because resourceType isn't one of the whitelisted types
-	_, err := NewAppDefinition([]byte(`acorns: {
+	_, err = NewAppDefinition([]byte(`acorns: {
 	s: {
     	labels: [
 			{
