@@ -93,26 +93,6 @@ func TestAcornProject(t *testing.T) {
 		wantOut        string
 		commandContext CommandContext
 	}{
-		{
-			name: "acorn image -j noproject",
-			fields: fields{
-				All:    false,
-				Quiet:  false,
-				Output: "",
-			},
-			commandContext: CommandContext{
-				ClientFactory: &testdata.MockClientFactory{},
-				StdOut:        w,
-				StdErr:        w,
-				StdIn:         strings.NewReader(""),
-			},
-			args: args{
-				args:   []string{"image", "-j", "noproject"},
-				client: &testdata.MockClient{},
-			},
-			wantErr: true,
-			wantOut: "projects.api.acorn.io \"noproject\" not found",
-		},
 		{
 			name: "acorn image -j 12.badname",
 			fields: fields{
@@ -133,26 +113,6 @@ func TestAcornProject(t *testing.T) {
 			wantErr: true,
 			wantOut: "invalid project name [12.badname]: can not contain \".\"",
 		},
-		{
-			name: "acorn image -j badname/projectname/here",
-			fields: fields{
-				All:    false,
-				Quiet:  false,
-				Output: "",
-			},
-			commandContext: CommandContext{
-				ClientFactory: &testdata.MockClientFactory{},
-				StdOut:        w,
-				StdErr:        w,
-				StdIn:         strings.NewReader(""),
-			},
-			args: args{
-				args:   []string{"image", "-j", "badname/projectname/here"},
-				client: &testdata.MockClient{},
-			},
-			wantErr: true,
-			wantOut: "failed to find authentication token for server badname, please run 'acorn login badname' first",
-		},
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
