@@ -12,7 +12,7 @@ import (
 func NewLogs(c CommandContext) *cobra.Command {
 	logs := &Logs{client: c.ClientFactory}
 	return cli.Command(logs, cobra.Command{
-		Use:               "logs [flags] [APP_NAME|CONTAINER_NAME]",
+		Use:               "logs [flags] [APP_NAME|CONTAINER_REPLICA_NAME]",
 		SilenceUsage:      true,
 		Short:             "Log all workloads from an app",
 		Args:              cobra.MaximumNArgs(1),
@@ -21,10 +21,11 @@ func NewLogs(c CommandContext) *cobra.Command {
 }
 
 type Logs struct {
-	Follow bool   `short:"f" usage:"Follow log output"`
-	Since  string `short:"s" usage:"Show logs since timestamp (e.g. 42m for 42 minutes)"`
-	Tail   int64  `short:"n" usage:"Number of lines in log output"`
-	client ClientFactory
+	Follow    bool   `short:"f" usage:"Follow log output"`
+	Since     string `short:"s" usage:"Show logs since timestamp (e.g. 42m for 42 minutes)"`
+	Tail      int64  `short:"n" usage:"Number of lines in log output"`
+	Container string `short:"c" usage:"Container name or Job name within app to follow"`
+	client    ClientFactory
 }
 
 func (s *Logs) Run(cmd *cobra.Command, args []string) error {
@@ -51,8 +52,9 @@ func (s *Logs) Run(cmd *cobra.Command, args []string) error {
 		tailLines = &s.Tail
 	}
 	return log.Output(cmd.Context(), c, args[0], &client.LogOptions{
-		Follow: s.Follow,
-		Tail:   tailLines,
-		Since:  s.Since,
+		Follow:    s.Follow,
+		Container: s.Container,
+		Tail:      tailLines,
+		Since:     s.Since,
 	})
 }
