@@ -1,6 +1,8 @@
 package cli
 
 import (
+	"sort"
+
 	cli "github.com/acorn-io/acorn/pkg/cli/builder"
 	"github.com/acorn-io/acorn/pkg/cli/builder/table"
 	"github.com/acorn-io/acorn/pkg/config"
@@ -26,6 +28,7 @@ acorn project`,
 	cmd.AddCommand(NewProjectCreate(c))
 	cmd.AddCommand(NewProjectRm(c))
 	cmd.AddCommand(NewProjectUse(c))
+	cmd.AddCommand(NewProjectUpdate(c))
 	return cmd
 }
 
@@ -36,8 +39,10 @@ type Project struct {
 }
 
 type projectEntry struct {
-	Name    string `json:"name,omitempty"`
-	Default bool   `json:"default,omitempty"`
+	Name          string   `json:"name,omitempty"`
+	Default       bool     `json:"default,omitempty"`
+	Regions       []string `json:"regions,omitempty"`
+	DefaultRegion string   `json:"default-region,omitempty"`
 }
 
 func (a *Project) Run(cmd *cobra.Command, args []string) error {
@@ -80,11 +85,33 @@ func (a *Project) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	out := table.NewWriter(tables.ProjectClient, a.Quiet, a.Output)
-	for _, project := range projectNames {
-		out.Write(projectEntry{
-			Name:    project,
-			Default: defaultProject == project,
-		})
+	projectDetails, err := project.GetDetails(cmd.Context(), a.client.Options(), projectNames)
+	if err != nil {
+		return err
+	}
+	sort.Slice(projectDetails, func(i, j int) bool {
+		return projectDetails[i].FullName < projectDetails[j].FullName
+	})
+
+	for _, projectItem := range projectDetails {
+		if projectItem.Err != nil {
+			logrus.Warnf("Could not list details of project [%s]: %v", projectItem.FullName, projectItem.Err)
+			continue
+		}
+		if projectItem.Project != nil {
+			supportedRegions := projectItem.Project.GetSupportedRegions()
+			defaultRegion := projectItem.Project.GetRegion()
+			for i, supportedRegion := range supportedRegions {
+				if supportedRegion == defaultRegion {
+					supportedRegions[i] = supportedRegion + "*"
+				}
+			}
+			out.Write(projectEntry{
+				Name:    projectItem.FullName,
+				Default: defaultProject == projectItem.FullName,
+				Regions: supportedRegions,
+			})
+		}
 	}
 
 	return out.Err()
