@@ -3,9 +3,12 @@ package cli
 import (
 	"fmt"
 	"strings"
+	"time"
 
 	"github.com/acorn-io/acorn/pkg/prompt"
 	"github.com/pterm/pterm"
+	"github.com/sirupsen/logrus"
+	apierrors "k8s.io/apimachinery/pkg/api/errors"
 
 	"github.com/acorn-io/acorn/pkg/client"
 	"github.com/spf13/cobra"
@@ -37,19 +40,13 @@ func addRmObject(rmObjects *RmObjects, obj string) {
 func getSecretsToRemove(arg string, client client.Client, cmd *cobra.Command) ([]string, error) {
 	var result []string
 	secrets, err := client.SecretList(cmd.Context())
-	apps, _ := client.AppList(cmd.Context())
-
 	if err != nil {
 		return nil, err
 	}
 
 	for _, secret := range secrets {
-		aliasList := aliases(&secret, apps)
-		if len(aliasList) != 0 {
-			secretName := strings.Split(aliasList[0], ".")
-			if len(secretName) != 0 && arg == secretName[0] {
-				result = append(result, secret.Name)
-			}
+		if strings.HasPrefix(secret.Name, arg+".") {
+			result = append(result, secret.Name)
 		}
 	}
 	return result, nil
@@ -62,10 +59,9 @@ func getVolumesToDelete(arg string, client client.Client, cmd *cobra.Command) ([
 	}
 
 	for _, volume := range volumes {
-		if arg == volume.Status.AppName {
+		if arg == volume.Status.AppName { // if the volume is a part of the app
 			result = append(result, volume.Name)
 		}
-
 	}
 	return result, nil
 }
@@ -108,8 +104,6 @@ func removeContainer(arg string, c client.Client, cmd *cobra.Command, force bool
 		}
 
 		fmt.Println("Removed: " + con)
-		continue
-
 	}
 	return nil
 }
@@ -146,11 +140,10 @@ func removeVolume(arg string, c client.Client, cmd *cobra.Command, force bool) e
 		} else {
 			fmt.Printf("Error: No such volume: %s\n", vol)
 		}
-
 	}
 	return nil
 }
-func removeApp(arg string, c client.Client, cmd *cobra.Command, force bool) error {
+func removeApp(arg string, c client.Client, cmd *cobra.Command, force, wait bool) error {
 	if !force {
 		pterm.FgRed.Println(arg)
 		err := prompt.Remove("app")
@@ -159,14 +152,36 @@ func removeApp(arg string, c client.Client, cmd *cobra.Command, force bool) erro
 		}
 	}
 	app, err := c.AppDelete(cmd.Context(), arg)
-
 	if err != nil {
 		return fmt.Errorf("deleting app %s: %w", arg, err)
 	}
-	if app != nil {
-		fmt.Println("Removed: " + arg)
-	} else {
+
+	if app == nil {
 		fmt.Printf("Error: No such app: %s\n", arg)
+		return nil
+	}
+
+	// Ensure this gets printed whether we wait or not
+	defer func() {
+		if err == nil || apierrors.IsNotFound(err) {
+			fmt.Println("Removed: " + arg)
+		}
+	}()
+	if wait {
+		fmt.Printf("Waiting for app %s to be removed...\n", arg)
+		for {
+			select {
+			case <-cmd.Context().Done():
+				return cmd.Context().Err()
+			default:
+				if _, err = c.AppGet(cmd.Context(), arg); apierrors.IsNotFound(err) {
+					return nil
+				} else if err != nil {
+					logrus.Debugf("Error getting app for removal check: %v", err)
+				}
+			}
+			time.Sleep(time.Second)
+		}
 	}
 	return nil
 }
