@@ -6,8 +6,8 @@ import (
 	"github.com/spf13/cobra"
 )
 
-func NewUninstall() *cobra.Command {
-	return cli.Command(&Uninstall{}, cobra.Command{
+func NewUninstall(c CommandContext) *cobra.Command {
+	return cli.Command(&Uninstall{client: c.ClientFactory}, cobra.Command{
 		Use: "uninstall [flags]",
 		Example: `
 # Uninstall with confirmation
@@ -22,8 +22,9 @@ acorn uninstall -f`,
 }
 
 type Uninstall struct {
-	Force bool `usage:"Do not prompt for confirmation" short:"f"`
-	All   bool `usage:"Delete all volumes and secrets" short:"a"`
+	Force  bool `usage:"Do not prompt for confirmation" short:"f"`
+	All    bool `usage:"Delete all volumes and secrets" short:"a"`
+	client ClientFactory
 }
 
 func (u *Uninstall) Run(cmd *cobra.Command, args []string) error {
