@@ -101,6 +101,20 @@ func (d *DeferredClient) AppPullImage(ctx context.Context, name string) error {
 	return d.Client.AppPullImage(ctx, name)
 }
 
+func (d *DeferredClient) DevSessionRenew(ctx context.Context, name string, client v1.DevSessionInstanceClient) error {
+	if err := d.create(); err != nil {
+		return err
+	}
+	return d.Client.DevSessionRenew(ctx, name, client)
+}
+
+func (d *DeferredClient) DevSessionRelease(ctx context.Context, name string) error {
+	if err := d.create(); err != nil {
+		return err
+	}
+	return d.Client.DevSessionRelease(ctx, name)
+}
+
 func (d *DeferredClient) CredentialCreate(ctx context.Context, serverAddress, username, password string, skipChecks bool) (*apiv1.Credential, error) {
 	if err := d.create(); err != nil {
 		return nil, err
@@ -206,6 +220,13 @@ func (d *DeferredClient) ContainerReplicaExec(ctx context.Context, name string,
 	return d.Client.ContainerReplicaExec(ctx, name, args, tty, opts)
 }
 
+func (d *DeferredClient) ContainerReplicaPortForward(ctx context.Context, containerName string, port int) (PortForwardDialer, error) {
+	if err := d.create(); err != nil {
+		return nil, err
+	}
+	return d.Client.ContainerReplicaPortForward(ctx, containerName, port)
+}
+
 func (d *DeferredClient) VolumeList(ctx context.Context) ([]apiv1.Volume, error) {
 	if err := d.create(); err != nil {
 		return nil, err
@@ -241,9 +262,9 @@ func (d *DeferredClient) ImageGet(ctx context.Context, name string) (*apiv1.Imag
 	return d.Client.ImageGet(ctx, name)
 }
 
-func (d *DeferredClient) ImageDelete(ctx context.Context, name string, opts *ImageDeleteOptions) (*apiv1.Image, error) {
+func (d *DeferredClient) ImageDelete(ctx context.Context, name string, opts *ImageDeleteOptions) (*apiv1.Image, []string, error) {
 	if err := d.create(); err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 	return d.Client.ImageDelete(ctx, name, opts)
 }
@@ -318,11 +339,18 @@ func (d *DeferredClient) ProjectList(ctx context.Context) ([]apiv1.Project, erro
 	return d.Client.ProjectList(ctx)
 }
 
-func (d *DeferredClient) ProjectCreate(ctx context.Context, name string) (*apiv1.Project, error) {
+func (d *DeferredClient) ProjectCreate(ctx context.Context, name, defaultRegion string, supportedRegions []string) (*apiv1.Project, error) {
 	if err := d.create(); err != nil {
 		return nil, err
 	}
-	return d.Client.ProjectCreate(ctx, name)
+	return d.Client.ProjectCreate(ctx, name, defaultRegion, supportedRegions)
+}
+
+func (d *DeferredClient) ProjectUpdate(ctx context.Context, project *apiv1.Project, defaultRegion string, supportedRegions []string) (*apiv1.Project, error) {
+	if err := d.create(); err != nil {
+		return nil, err
+	}
+	return d.Client.ProjectUpdate(ctx, project, defaultRegion, supportedRegions)
 }
 
 func (d *DeferredClient) ProjectDelete(ctx context.Context, name string) (*apiv1.Project, error) {
@@ -367,6 +395,27 @@ func (d *DeferredClient) VolumeClassGet(ctx context.Context, name string) (*apiv
 	return d.Client.VolumeClassGet(ctx, name)
 }
 
+func (d *DeferredClient) RegionList(ctx context.Context) ([]apiv1.Region, error) {
+	if err := d.create(); err != nil {
+		return nil, err
+	}
+	return d.Client.RegionList(ctx)
+}
+
+func (d *DeferredClient) RegionGet(ctx context.Context, name string) (*apiv1.Region, error) {
+	if err := d.create(); err != nil {
+		return nil, err
+	}
+	return d.Client.RegionGet(ctx, name)
+}
+
+func (d *DeferredClient) EventStream(ctx context.Context, opts *EventStreamOptions) (<-chan apiv1.Event, error) {
+	if err := d.create(); err != nil {
+		return nil, err
+	}
+	return d.Client.EventStream(ctx, opts)
+}
+
 func (d *DeferredClient) GetProject() string {
 	return d.Project
 }
