@@ -6,7 +6,6 @@ import (
 	"strings"
 
 	apiv1 "github.com/acorn-io/acorn/pkg/apis/api.acorn.io/v1"
-	v1 "github.com/acorn-io/acorn/pkg/apis/internal.acorn.io/v1"
 	"github.com/acorn-io/acorn/pkg/system"
 	"github.com/acorn-io/baaah/pkg/router"
 	corev1 "k8s.io/api/core/v1"
@@ -17,11 +16,10 @@ import (
 )
 
 var (
-	ClusterDomainDefault         = ".local.on-acorn.io"
+	ClusterDomainDefault         = ".local.oss-acorn.io"
 	InternalClusterDomainDefault = "svc.cluster.local"
 
-	// AcornDNSEndpointDefault will be overridden at build time for releases
-	AcornDNSEndpointDefault = "https://staging-dns.acrn.io/v1"
+	AcornDNSEndpointDefault = "https://oss-dns.acrn.io/v1"
 	AcornDNSStateDefault    = "auto"
 
 	// LetsEncryptOptionDefault is the default state for the Let's Encrypt integration
@@ -32,12 +30,15 @@ var (
 
 	// Default HttpEndpointPattern set to enable Let's Encrypt
 	DefaultHttpEndpointPattern = "{{hashConcat 8 .Container .App .Namespace | truncate}}.{{.ClusterDomain}}"
+
+	// Features
+	FeatureImageAllowRules = "image-allow-rules"
+	FeatureDefaults        = map[string]bool{
+		FeatureImageAllowRules: false,
+	}
 )
 
 func complete(ctx context.Context, c *apiv1.Config, getter kclient.Reader) error {
-	if len(c.DefaultPublishMode) == 0 {
-		c.DefaultPublishMode = v1.PublishModeDefined
-	}
 	if c.SetPodSecurityEnforceProfile == nil {
 		c.SetPodSecurityEnforceProfile = &[]bool{true}[0]
 	}
@@ -96,7 +97,24 @@ func complete(ctx context.Context, c *apiv1.Config, getter kclient.Reader) error
 	if c.UseCustomCABundle == nil {
 		c.UseCustomCABundle = new(bool)
 	}
-
+	if c.NetworkPolicies == nil {
+		c.NetworkPolicies = &[]bool{true}[0]
+	}
+	if c.IngressControllerNamespace == nil {
+		c.IngressControllerNamespace = new(string)
+	}
+	if c.AWSIdentityProviderARN == nil {
+		c.AWSIdentityProviderARN = new(string)
+	}
+	if c.Features == nil {
+		c.Features = FeatureDefaults
+	} else {
+		for k, v := range FeatureDefaults {
+			if _, ok := c.Features[k]; !ok {
+				c.Features[k] = v
+			}
+		}
+	}
 	return nil
 }
 
@@ -146,8 +164,9 @@ func setClusterDomains(ctx context.Context, c *apiv1.Config, getter kclient.Read
 		}
 	}
 
-	// If a clusterDomain hasn't been set yet, use the localhost wildcard domain
-	if len(c.ClusterDomains) == 0 {
+	// If a clusterDomain hasn't been set yet and acorn-dns hasn't been explicitly disabled,
+	// use the localhost wildcard domain
+	if len(c.ClusterDomains) == 0 && !strings.EqualFold(*c.AcornDNS, "disabled") {
 		c.ClusterDomains = []string{ClusterDomainDefault}
 	}
 	return nil
@@ -159,7 +178,7 @@ func useLocalWildcardDomain(ctx context.Context, getter kclient.Reader) (bool, e
 	var nodes corev1.NodeList
 	if err := getter.List(ctx, &nodes); err != nil {
 		if meta.IsNoMatchError(err) {
-			// Node type doesn't exist probably because we are running against huhb
+			// Node type doesn't exist probably because we are running against hub
 			return false, nil
 		}
 		return false, err
@@ -199,6 +218,14 @@ func IsDockerDesktop(ctx context.Context, getter kclient.Reader) (bool, error) {
 	return false, nil
 }
 
+// merge merges two Config objects. The newConfig object takes precedence over the oldConfig object.
+//
+// WARNING: We have had many bugs with this merge logic. To avoid this when adding fields here, there
+// are two main cases to be considered when adding a new field to the Config object and merging it here:
+//
+// 1. If the newConfig does not pass a field at all, the field in the oldConfig should be used.
+// 2. The newConfig should have a way of unsetting the values in the oldConfig.
+// 3. A newConfig should have a way of setting values that overwrite the oldConfig.
 func merge(oldConfig, newConfig *apiv1.Config) *apiv1.Config {
 	var (
 		mergedConfig apiv1.Config
@@ -258,9 +285,6 @@ func merge(oldConfig, newConfig *apiv1.Config) *apiv1.Config {
 		}
 		mergedConfig.ClusterDomains = newConfig.ClusterDomains
 	}
-	if len(newConfig.DefaultPublishMode) > 0 {
-		mergedConfig.DefaultPublishMode = newConfig.DefaultPublishMode
-	}
 	if newConfig.HttpEndpointPattern != nil {
 		mergedConfig.HttpEndpointPattern = newConfig.HttpEndpointPattern
 	}
@@ -304,6 +328,10 @@ func merge(oldConfig, newConfig *apiv1.Config) *apiv1.Config {
 		mergedConfig.UseCustomCABundle = newConfig.UseCustomCABundle
 	}
 
+	if newConfig.Features != nil {
+		mergedConfig.Features = newConfig.Features
+	}
+
 	if len(newConfig.PropagateProjectAnnotations) > 0 && newConfig.PropagateProjectAnnotations[0] == "" {
 		mergedConfig.PropagateProjectAnnotations = nil
 	} else if len(newConfig.PropagateProjectAnnotations) > 0 {
@@ -316,6 +344,34 @@ func merge(oldConfig, newConfig *apiv1.Config) *apiv1.Config {
 		mergedConfig.PropagateProjectLabels = newConfig.PropagateProjectLabels
 	}
 
+	if len(newConfig.AllowTrafficFromNamespace) > 0 && newConfig.AllowTrafficFromNamespace[0] == "" {
+		mergedConfig.AllowTrafficFromNamespace = nil
+	} else if len(newConfig.AllowTrafficFromNamespace) > 0 {
+		mergedConfig.AllowTrafficFromNamespace = newConfig.AllowTrafficFromNamespace
+	}
+
+	if len(newConfig.ServiceLBAnnotations) > 0 && newConfig.ServiceLBAnnotations[0] == "" {
+		mergedConfig.ServiceLBAnnotations = nil
+	} else if len(newConfig.ServiceLBAnnotations) > 0 {
+		mergedConfig.ServiceLBAnnotations = newConfig.ServiceLBAnnotations
+	}
+
+	if newConfig.NetworkPolicies != nil {
+		mergedConfig.NetworkPolicies = newConfig.NetworkPolicies
+	}
+
+	if newConfig.IngressControllerNamespace != nil {
+		mergedConfig.IngressControllerNamespace = newConfig.IngressControllerNamespace
+	}
+
+	if newConfig.AWSIdentityProviderARN != nil {
+		mergedConfig.AWSIdentityProviderARN = newConfig.AWSIdentityProviderARN
+	}
+
+	if newConfig.EventTTL != nil {
+		mergedConfig.EventTTL = newConfig.EventTTL
+	}
+
 	return &mergedConfig
 }
 
