@@ -4,11 +4,13 @@ import (
 	"crypto/sha256"
 	"encoding/hex"
 	"encoding/json"
+	"errors"
 	"net/url"
 	"path"
 	"strings"
 
 	v1 "github.com/acorn-io/acorn/pkg/apis/internal.acorn.io/v1"
+	"github.com/acorn-io/acorn/pkg/appdefinition"
 	"github.com/acorn-io/acorn/pkg/condition"
 	"github.com/acorn-io/acorn/pkg/config"
 	"github.com/acorn-io/acorn/pkg/images"
@@ -29,6 +31,7 @@ import (
 	apierror "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/intstr"
+	"k8s.io/apimachinery/pkg/util/sets"
 	kclient "sigs.k8s.io/controller-runtime/pkg/client"
 )
 
@@ -51,11 +54,6 @@ func FilterLabelsAndAnnotationsConfig(h router.Handler) router.Handler {
 }
 
 func DeploySpec(req router.Request, resp router.Response) (err error) {
-	// We need this handler to run even when the app is deleting so ensure jobs will clean up
-	if req.Object == nil {
-		return nil
-	}
-
 	appInstance := req.Object.(*v1.AppInstance)
 	status := condition.Setter(appInstance, resp, v1.AppInstanceConditionDefined)
 	interpolator := secrets.NewInterpolator(req.Ctx, req.Client, appInstance)
@@ -68,6 +66,11 @@ func DeploySpec(req router.Request, resp router.Response) (err error) {
 			// because it's failing. This happens when we refer to an object that has
 			// yet to be created
 			status.Error(interpolator.Err())
+		} else if errors.Is(err, appdefinition.ErrInvalidInput) {
+			status.Error(err)
+			// Not all object could have been rendered to don't purge anything
+			resp.DisablePrune()
+			err = nil
 		} else {
 			status.Error(err)
 		}
@@ -92,7 +95,7 @@ func DeploySpec(req router.Request, resp router.Response) (err error) {
 	if err := addJobs(req, appInstance, tag, pullSecrets, interpolator, resp); err != nil {
 		return err
 	}
-	if err := addServices(req, appInstance, resp); err != nil {
+	if err := addServices(req, appInstance, interpolator, resp); err != nil {
 		return err
 	}
 	if err := addPVCs(req, appInstance, resp); err != nil {
@@ -130,14 +133,27 @@ func toEnvFrom(envs []v1.EnvVar) (result []corev1.EnvFromSource) {
 	return
 }
 
-func toEnv(envs []v1.EnvVar, appEnv []v1.NameValue, interpolator *secrets.Interpolator) (result []corev1.EnvVar) {
+func toEnv(envs []v1.EnvVar, appEnvs []v1.NameValue, interpolator *secrets.Interpolator) (result []corev1.EnvVar) {
+	appEnvNames := sets.New[string]()
+	for _, appEnv := range appEnvs {
+		if appEnv.Name != "" {
+			appEnvNames.Insert(appEnv.Name)
+		}
+	}
+
 	for _, env := range envs {
 		if env.Secret.Name == "" {
+			if appEnvNames.Has(env.Name) {
+				continue
+			}
 			result = append(result, interpolator.ToEnv(env.Name, env.Value))
 		} else {
 			if env.Secret.Key == "" {
 				continue
 			}
+			if appEnvNames.Has(env.Name) {
+				continue
+			}
 			result = append(result, corev1.EnvVar{
 				Name: env.Name,
 				ValueFrom: &corev1.EnvVarSource{
@@ -151,11 +167,8 @@ func toEnv(envs []v1.EnvVar, appEnv []v1.NameValue, interpolator *secrets.Interp
 			})
 		}
 	}
-	for _, appEnv := range appEnv {
-		result = append(result, corev1.EnvVar{
-			Name:  appEnv.Name,
-			Value: appEnv.Value,
-		})
+	for _, appEnv := range appEnvs {
+		result = append(result, interpolator.ToEnv(appEnv.Name, appEnv.Value))
 	}
 	return
 }
@@ -553,7 +566,9 @@ func getSecretAnnotations(req router.Request, appInstance *v1.AppInstance, conta
 		}
 		rev, err := getRevision(req, appInstance.Status.Namespace, secret)
 		if apierror.IsNotFound(err) {
-			result[apply.AnnotationUpdate] = "false"
+			if !appInstance.GetStopped() {
+				result[apply.AnnotationUpdate] = "false"
+			}
 			result[apply.AnnotationCreate] = "false"
 		} else if err != nil {
 			return nil, err
