@@ -1,12 +1,15 @@
 package appdefinition
 
 import (
+	"encoding/json"
+	"fmt"
+
 	v1 "github.com/acorn-io/acorn/pkg/apis/internal.acorn.io/v1"
 	"github.com/acorn-io/acorn/pkg/awspermissions"
+	"github.com/acorn-io/acorn/pkg/labels"
 	"github.com/acorn-io/baaah/pkg/router"
 	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"k8s.io/apimachinery/pkg/labels"
 	kclient "sigs.k8s.io/controller-runtime/pkg/client"
 )
 
@@ -37,6 +40,17 @@ func addAWS(req router.Request, appInstance *v1.AppInstance, sa *corev1.ServiceA
 		return err
 	}
 
+	if perms.HasRules() {
+		data, err := json.Marshal(perms)
+		if err != nil {
+			return fmt.Errorf("marshaling permission rules: %v", err)
+		}
+		if annotations == nil {
+			annotations = map[string]string{}
+		}
+		annotations[labels.AcornPermissions] = string(data)
+	}
+
 	sa.Annotations = labels.Merge(sa.Annotations, annotations)
 	return nil
 }
