@@ -26,9 +26,9 @@ func TestToEndpoint(t *testing.T) {
 		wantErr      error
 	}{
 		{
-			name: "\"on-acorn.io no -\" pattern set",
+			name: "\"oss-acorn.io no -\" pattern set",
 			args: args{
-				domain:      "domain.on-acorn.io",
+				domain:      "domain.oss-acorn.io",
 				serviceName: "app-test",
 				pattern:     "{{hashConcat 8 .Container .App .Namespace | truncate}}.{{.ClusterDomain}}",
 				appInstance: &v1.AppInstance{
@@ -38,12 +38,12 @@ func TestToEndpoint(t *testing.T) {
 					Status:     v1.AppInstanceStatus{},
 				},
 			},
-			wantEndpoint: "app-test-green-star-b19d0b34.domain.on-acorn.io",
+			wantEndpoint: "app-test-green-star-b19d0b34.domain.oss-acorn.io",
 		},
 		{
-			name: "\"on-acorn.io no -\" pattern set with 12 characters",
+			name: "\"oss-acorn.io no -\" pattern set with 12 characters",
 			args: args{
-				domain:      "domain.on-acorn.io",
+				domain:      "domain.oss-acorn.io",
 				serviceName: "app-test",
 				pattern:     "{{hashConcat 12 .Container .App .Namespace | truncate}}.{{.ClusterDomain}}",
 				appInstance: &v1.AppInstance{
@@ -53,12 +53,12 @@ func TestToEndpoint(t *testing.T) {
 					Status:     v1.AppInstanceStatus{},
 				},
 			},
-			wantEndpoint: "app-test-green-star-b19d0b346674.domain.on-acorn.io",
+			wantEndpoint: "app-test-green-star-b19d0b346674.domain.oss-acorn.io",
 		},
 		{
-			name: "\"on-acorn.io no -\" pattern set with less than two parameters should return empty string",
+			name: "\"oss-acorn.io no -\" pattern set with less than two parameters should return empty string",
 			args: args{
-				domain:      "domain.on-acorn.io",
+				domain:      "domain.oss-acorn.io",
 				serviceName: "app-test",
 				pattern:     "{{hashConcat 8 .Container | truncate}}.{{.ClusterDomain}}",
 				appInstance: &v1.AppInstance{
@@ -68,7 +68,7 @@ func TestToEndpoint(t *testing.T) {
 					Status:     v1.AppInstanceStatus{},
 				},
 			},
-			wantEndpoint: ".domain.on-acorn.io",
+			wantEndpoint: ".domain.oss-acorn.io",
 		},
 		{
 			name: "\"custom domain\" pattern set",
@@ -210,13 +210,13 @@ func TestToEndpoint(t *testing.T) {
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			gotEndpoint, err := toEndpoint(tt.args.pattern, tt.args.domain, tt.args.serviceName, tt.args.appInstance.GetName(), tt.args.appInstance.GetNamespace())
+			gotEndpoint, err := toHTTPEndpointHostname(tt.args.pattern, tt.args.domain, tt.args.serviceName, tt.args.appInstance.GetName(), tt.args.appInstance.GetNamespace())
 			if !errors.Is(err, tt.wantErr) {
-				t.Fatalf("toEndpoint() error = %v, want %v", err, tt.wantErr)
+				t.Fatalf("toHTTPEndpointHostname() error = %v, want %v", err, tt.wantErr)
 			}
 
 			if gotEndpoint != tt.wantEndpoint {
-				t.Errorf("toEndpoint() = %v, want %v", gotEndpoint, tt.wantEndpoint)
+				t.Errorf("toHTTPEndpointHostname() = %v, want %v", gotEndpoint, tt.wantEndpoint)
 			}
 		})
 	}
@@ -258,7 +258,7 @@ func TestValidateEndpointPattern(t *testing.T) {
 		t.Run(tt.name, func(t *testing.T) {
 			err := ValidateEndpointPattern(tt.pattern)
 			if !errors.Is(err, tt.wantErr) {
-				t.Fatalf("toEndpoint() error = %v, want %v", err, tt.wantErr)
+				t.Fatalf("toHTTPEndpointHostname() error = %v, want %v", err, tt.wantErr)
 			}
 		})
 	}
@@ -286,7 +286,7 @@ func Test_setupCertManager(t *testing.T) {
 			want: []networkingv1.IngressTLS{{Hosts: []string{"host"}}},
 		},
 		{
-			name: "annotation but tls found",
+			name: "annotation and tls found, prefer annotation",
 			args: args{
 				serviceName: "foo",
 				annotations: map[string]string{
@@ -295,7 +295,7 @@ func Test_setupCertManager(t *testing.T) {
 				rules: []networkingv1.IngressRule{{Host: "host1"}},
 				tls:   []networkingv1.IngressTLS{{Hosts: []string{"host"}}},
 			},
-			want: []networkingv1.IngressTLS{{Hosts: []string{"host"}}},
+			want: []networkingv1.IngressTLS{{Hosts: []string{"host1"}, SecretName: "foo-cm-cert-1"}},
 		},
 		{
 			name: "cluster-issuer annotation found",
