@@ -24,12 +24,13 @@ var (
 				Verbs: []string{"get", "list"},
 				Resources: []string{
 					"projects",
+					"regions",
 				},
 			},
 		},
 		ClusterEdit: {
 			{
-				Verbs: []string{"create", "delete"},
+				Verbs: []string{"create", "update", "delete"},
 				Resources: []string{
 					"projects",
 				},
@@ -44,11 +45,14 @@ var (
 					"apps",
 					"acornimagebuilds",
 					"builders",
+					"devsessions",
 					"images",
 					"volumes",
 					"containerreplicas",
 					"credentials",
 					"secrets",
+					"services",
+					"events",
 				},
 			},
 			{
@@ -56,6 +60,8 @@ var (
 				Resources: []string{
 					"volumeclasses",
 					"computeclasses",
+					"regions",
+					"imageallowrules",
 				},
 			},
 			{
@@ -84,6 +90,7 @@ var (
 				Verbs: []string{"create", "update", "delete", "patch"},
 				Resources: []string{
 					"apps",
+					"devsessions",
 					"credentials",
 					"secrets",
 				},
@@ -105,6 +112,7 @@ var (
 			{
 				Verbs: []string{"delete"},
 				Resources: []string{
+					"services",
 					"volumes",
 					"containerreplicas",
 				},
@@ -145,6 +153,12 @@ var (
 				},
 				APIGroups: []string{admin_acorn_io.Group},
 			},
+			{
+				Verbs: []string{"*"},
+				Resources: []string{
+					"imageallowrules",
+				},
+			},
 		},
 	}
 )
@@ -194,7 +208,7 @@ func ClusterRoles() []rbacv1.ClusterRole {
 			ObjectMeta: metav1.ObjectMeta{
 				Name: Admin,
 			},
-			Rules: concat(View, ViewLogs, Edit, Build),
+			Rules: concat(View, ViewLogs, Edit, Build, Admin),
 		},
 		{
 			ObjectMeta: metav1.ObjectMeta{
